define(function (require, exports, module) {
	'use strict';

	var NodeSet = require('ft/core/nodeset').NodeSet,
		Extensions = require('ft/core/extensions').Extensions,
		DateUtils = require('ft/util/date'),
		viewportClocks = new NodeSet(),
		date = new Date();

	function formatDate(clockNode) {
		var formatProperty = clockNode.propertyNode('format'),
			format;

		if (formatProperty) {
			format = formatProperty.tag('value');
		} else {
			format = 'shortTime';
		}

		return date.format(format);
	}

	Extensions.addMode({
		name: 'clock'
	});

	Extensions.addRenderNode(function (editor, node, nodeRenderer) {
		if (node.mode() === 'clock') {
			var widget = document.createElement('span');
			widget.textContent = formatDate(node);
			nodeRenderer.renderLineWidget(widget, {
				overlay: true,
				positionWidget : function (widgetWidth, widgetHeight) {
					var line = node.lineNumber(),
						leadingSpace = node.line().match(/\s*/)[0],
						coords = editor.cursorCoords({ line : line, ch : leadingSpace.length}, 'div');
					return {
						left: Math.round(coords.left - (widgetWidth + editor.defaultSpaceWidth())) + 'px'
					};
				}
			});
		}
	});

	Extensions.add('com.foldingtext.editor.viewportNodesChanged', function (editor, nodes, removedNodes, addedNodes) {
		viewportClocks.minusSet(removedNodes);
		addedNodes.forEachNodeInSet(function (node) {
			if (node.mode() === 'clock') {
				viewportClocks.addNode(node);
			}
		});
	});

	Extensions.add('com.foldingtext.editor.clockTick', function (editor, clockDate) {
		date = clockDate;
		viewportClocks.forEachNodeInSet(function (each) {
			if (each.mode() !== 'clock') {
				viewportClocks.removeNode(each);
			} else {
				editor.setNeedsRender(each);
			}
		});
	});
});