//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
    'use strict';

	require('ft/extensions/undo');
	require('ft/extensions/mouse');
	require('ft/extensions/indent');
	require('ft/extensions/current');
	require('ft/extensions/complete');
	require('ft/extensions/movebranch');
	require('ft/extensions/moveparagraph');
	require('ft/extensions/smartnewline');
	require('ft/extensions/folding');
	require('ft/extensions/snippets');
	require('ft/extensions/print');
	require('ft/extensions/focus');
	require('ft/extensions/foldingnodes');
	require('ft/extensions/selection');
	require('ft/extensions/dragdrop');
	require('ft/extensions/cutcopypaste');
	require('ft/extensions/textchecking');
	require('ft/extensions/formatting');
	require('ft/extensions/localstorage');

	var theme = require('ft/editor/theme'),
		DateUtils = require('ft/util/date'),
		Extensions = require('ft/core/extensions').Extensions,
		queryParameter = require('ft/util/queryparameter').QueryParameter,
		pluginPaths = queryParameter('pluginPaths', '').split(':'),
		runningInBrowser = !window.nativeViewController;

	function initEditor(taxonomy, appstyles) {
		if (!runningInBrowser) {
			require('ft/util/environment').disableLocalStorage = true;
		}

		theme.loadLESS(taxonomy.styles(), appstyles, function () {
			var text = '',
				tabSize = theme.readNumberStyleVariable('@tabSize', 4),
				Editor = require('ft/editor/editor').Editor,
				options = {
					tabIndex: 1,
					autofocus: true,
					lineNumbers: false,
					lineWrapping: true,
					indentWithTabs: true,
					flattenSpans: true,
					tabSize: tabSize,
					indentUnit: tabSize,
					enableCompositionMod: true,
					viewportMargin: 5,
					historyEventDelay: 1250,
					undoDepth: Infinity,

					// can cause weird jiggle when focusing input
					// element and input element is scrolled
					// offscreen, so set to false. But turnning off
					// breaks location composing input popup
					moveInputWithCursor: false, 

					mode: null,
					theme: '',
					value: ''
				},
				overrideOptions = JSON.parse(queryParameter('options', '{}'));

			for (var each in overrideOptions) {
				options[each] = overrideOptions[each];
			}

			new Editor(text, taxonomy, options);
		});

		Extensions.addInit(function (editor) {
			editor.addKeyMap({
				name: 'smartNewLine' ,
				'Enter': 'smartInsertNewline',
				'Tab': 'smartInsertTab',
				'Shift-Cmd-Enter' : 'smartInsertNewlineAbove',
				'Cmd-Enter' : 'smartInsertNewlineAfter'
			});

			if (runningInBrowser) {
				editor.addKeyMap({
					name: 'webbindings',
					'Cmd-Z' : 'undo',
					'Shift-Cmd-Z' : 'redo',

					'Cmd-A' : 'selectAll',

					// Indent
					'Shift-Tab': 'moveParagraphsLeft',
					'Tab': 'smartInsertTab',
					'Cmd-[': 'moveParagraphsLeft',
					'Cmd-]': 'moveParagraphsRight',
					'Cmd-Alt-[': 'moveParagraphsUp',
					'Cmd-Alt-]': 'moveParagraphsDown',

					// Branches
					'Ctrl-Alt-Left': 'moveBranchesLeft',
					'Ctrl-Alt-Right': 'moveBranchesRight',
					'Ctrl-Alt-Up': 'moveBranchesUp',
					'Ctrl-Alt-Down': 'moveBranchesDown',

					// Selection
					'Cmd-Alt-Up': 'expandSelection',
					'Cmd-Alt-Down': 'contractSelection',

					// Formatting
					'Cmd-B': 'toggleBold',
					'Cmd-I': 'toggleItalic',
					'Cmd-L': 'formatUnorderedList',
					'Shift-Cmd-L': 'formatOrderedList',

					// Folding
					'Cmd-/': 'toggleNodeFold',
					'Cmd-Alt-/': 'toggleNodeFoldNodeAndDescendants',
					'Cmd-Alt-1': function (editor) {
						editor.performCommand('foldingLevel', {interfaceItemTag: 1});
					},
					'Cmd-Alt-2': function (editor) {
						editor.performCommand('foldingLevel', {interfaceItemTag: 2});
					},
					'Cmd-Alt-3': function (editor) {
						editor.performCommand('foldingLevel', {interfaceItemTag: 3});
					},
					'Cmd-Alt-4': function (editor) {
						editor.performCommand('foldingLevel', {interfaceItemTag: 4});
					},

					'Cmd-U': 'focusIn',

					'Cmd-Alt-Right': 'expandFoldingLevel',
					'Cmd-Alt-Left': 'contractFoldingLevel',
					'Shift-Cmd-A': 'home'
				});
			}
		});		
	}

	return {
		app : function (taxonomy, appstyles) {			
			require(pluginPaths, function () {
				initEditor(taxonomy, appstyles);
			}, function (err) {
				if (window.nativeViewController) {
					window.nativeViewController.javascriptReportPluginError_(JSON.stringify({
						line: err.line,
						message: err.message,
						sourceURL: err.sourceURL
					}));									
				}
				console.log('Error requiring plugin paths', err);
				initEditor(taxonomy, appstyles);
			});
		}
	};
});