//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//

// For node.js environments
if ('undefined' === typeof window) {
	window = {};
}

/** @module codemirror */
define(function (require, exports, module) {
	'use strict';

	// Convienence to import ALL of codemirror. Makes it easier to handle cases like
	// codemirrormode helper, where we only want to load in all the codemirror modes if
	// codemirror itself has been defined (since we don't want codemirror and the modes
	// loaded into command line app.

	var CodeMirror = require('components/codemirror/lib/codemirror');

	require('components/codemirror/addon/dialog/dialog');
	require('components/codemirror/addon/display/rulers');
	require('components/codemirror/addon/selection/active-line');
	// remove this, creates oddities in text kerning since extra spans are
	//added require('components/codemirror/addon/selection/mark-selection');
	require('components/codemirror/addon/search/match-highlighter');
	require('components/codemirror/addon/search/search');
	require('components/codemirror/addon/search/searchcursor');

	require('components/codemirror/keymap/emacs');
	require('components/codemirror/keymap/sublime');
	require('components/codemirror/keymap/vim');

	require('components/codemirror/mode/apl/apl');
	require('components/codemirror/mode/asterisk/asterisk');
	require('components/codemirror/mode/clike/clike');
	require('components/codemirror/mode/clojure/clojure');
	require('components/codemirror/mode/cobol/cobol');
	require('components/codemirror/mode/coffeescript/coffeescript');
	require('components/codemirror/mode/commonlisp/commonlisp');
	require('components/codemirror/mode/css/css');
	require('components/codemirror/mode/d/d');
	require('components/codemirror/mode/diff/diff');
	require('components/codemirror/mode/dtd/dtd');
	require('components/codemirror/mode/ecl/ecl');
	require('components/codemirror/mode/eiffel/eiffel');
	require('components/codemirror/mode/erlang/erlang');
	require('components/codemirror/mode/fortran/fortran');
	require('components/codemirror/mode/gas/gas');
	require('components/codemirror/mode/gfm/gfm');
	require('components/codemirror/mode/gherkin/gherkin');
	require('components/codemirror/mode/go/go');
	require('components/codemirror/mode/groovy/groovy');
	require('components/codemirror/mode/haml/haml');
	require('components/codemirror/mode/haskell/haskell');
	require('components/codemirror/mode/haxe/haxe');
	require('components/codemirror/mode/htmlembedded/htmlembedded');
	require('components/codemirror/mode/htmlmixed/htmlmixed');
	require('components/codemirror/mode/http/http');
	require('components/codemirror/mode/jade/jade');
	require('components/codemirror/mode/javascript/javascript');
	require('components/codemirror/mode/jinja2/jinja2');
	require('components/codemirror/mode/julia/julia');
	require('components/codemirror/mode/livescript/livescript');
	require('components/codemirror/mode/lua/lua');
	require('components/codemirror/mode/markdown/markdown');
	require('components/codemirror/mode/mirc/mirc');
	require('components/codemirror/mode/mllike/mllike');
	require('components/codemirror/mode/nginx/nginx');
	require('components/codemirror/mode/ntriples/ntriples');
	require('components/codemirror/mode/octave/octave');
	require('components/codemirror/mode/pascal/pascal');
	require('components/codemirror/mode/pegjs/pegjs');
	require('components/codemirror/mode/perl/perl');
	require('components/codemirror/mode/php/php');
	require('components/codemirror/mode/pig/pig');
	require('components/codemirror/mode/properties/properties');
	require('components/codemirror/mode/puppet/puppet');
	require('components/codemirror/mode/python/python');
	require('components/codemirror/mode/q/q');
	require('components/codemirror/mode/r/r');
	require('components/codemirror/mode/rpm/rpm');
	require('components/codemirror/mode/rst/rst');
	require('components/codemirror/mode/ruby/ruby');
	require('components/codemirror/mode/rust/rust');
	require('components/codemirror/mode/sass/sass');
	require('components/codemirror/mode/scheme/scheme');
	require('components/codemirror/mode/shell/shell');
	require('components/codemirror/mode/sieve/sieve');
	require('components/codemirror/mode/smalltalk/smalltalk');
	require('components/codemirror/mode/smarty/smarty');
	require('components/codemirror/mode/smartymixed/smartymixed');
	require('components/codemirror/mode/solr/solr');
	require('components/codemirror/mode/sparql/sparql');
	require('components/codemirror/mode/sql/sql');
	require('components/codemirror/mode/stex/stex');
	require('components/codemirror/mode/tcl/tcl');
	require('components/codemirror/mode/tiddlywiki/tiddlywiki');
	require('components/codemirror/mode/tiki/tiki');
	require('components/codemirror/mode/toml/toml');
	require('components/codemirror/mode/turtle/turtle');
	require('components/codemirror/mode/vb/vb');
	require('components/codemirror/mode/vbscript/vbscript');
	require('components/codemirror/mode/velocity/velocity');
	require('components/codemirror/mode/verilog/verilog');
	require('components/codemirror/mode/xml/xml');
	require('components/codemirror/mode/xquery/xquery');
	require('components/codemirror/mode/yaml/yaml');
	require('components/codemirror/mode/z80/z80');

	require('components/codemirror/mode/meta');

	var CodeMirrorCSS = require('components/text/text!components/codemirror/lib/codemirror.css');

	CodeMirrorCSS += '\n\n\n' + require('components/text/text!components/codemirror/addon/dialog/dialog.css');

	exports.CodeMirrorCSS = CodeMirrorCSS;

	/**
	 * {@tutorial CodeMirror} namespace object. To get the CodeMirror editor instance use
	 * {@link module:ft/editor/editor.Editor#cm|editor.cm()}.
	 *
	 * @private
	 * @constructor
	 * @tutorial Nodes
	 * @alias module:codemirror.CodeMirror
	 */
	exports.CodeMirror = CodeMirror;
});