//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var ObjectUtils = require('../util/objectutils');

	var AttributeRun = function AttributeRun(location, length, attributes) {
		this.location = location;
		this.length = length;
		this.attributes = attributes;
	};

	(function () {

		this.copy = function copy() {
			return new AttributeRun(this.location, this.length, ObjectUtils.shallowClone(this.attributes));
		};

		this.splitAtIndex = function splitAtIndex(index) {
			var location = this.location,
				length = this.length,
				end = location + length;

			var newAttributeRun = new AttributeRun(
				index,
				(location + length) - index,
				index === end ? {} : ObjectUtils.shallowClone(this.attributes)
			);
			this.length = index - location;
			return newAttributeRun;
		};

		this.toString = function toString() {
			var attributes = this.attributes,
				sortedNames = [],
				nameValues = [],
				each,
				name,
				i;

			for (each in this.attributes) {
				sortedNames.push(each);
			}

			sortedNames.sort();

			var length = sortedNames.length;
			for (i = 0; i < length; i++) {
				name = sortedNames[i];
				nameValues.push(name + '="' + attributes[name] + '"');
			}

			return this.location + ',' + this.length + '/' + nameValues.join('/');
		};

		this._mergeWithNext = function _mergeWithNext(attributeRun) {
			if (this.location + this.length === attributeRun.location) {
				if (ObjectUtils.shallowEquals(this.attributes, attributeRun.attributes)) {
					this.length += attributeRun.length;
					return true;
				}
			}
			return false;
		};		

	}).call(AttributeRun.prototype);

	exports.AttributeRun = AttributeRun;
});