//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var Classifier = require('./classifier').Classifier,
		Extensions = require('./extensions').Extensions,
		Taxonomy = require('./taxonomy').Taxonomy;

	function gatherTaxonomyExtensions() {
		var taxonomyExtensions = {},
			mimeTaxonomies = {},
			taxonomies = {};

		Extensions.processExtensions('com.foldingtext.editor.taxonomies', function (each) {
			var configurations = each.configurations,
				length = configurations.length;
			for (var i = 0; i < length; i++) {
				mimeTaxonomies[configurations[i].mime] = configurations[i].spec;
			}
			taxonomies[each.name] = each.taxonomy;
		});

		return {
			taxonomyExtensions: taxonomyExtensions,
			mimeTaxonomies: mimeTaxonomies,
			taxonomies: taxonomies
		};
	}

	function resolveTaxonomy(spec, extensions) {
		if (typeof spec === 'string' && extensions.mimeTaxonomies.hasOwnProperty(spec)) {
			spec = extensions.mimeTaxonomies[spec];
		} else if (typeof spec === 'string' && /^[\w\-]+\/[\w\-]+\+xml$/.test(spec)) {
			return resolveTaxonomy('application/xml', extensions);
		}

		if (typeof spec === 'string') {
			return {name: spec};
		} else {
			return spec || {name: 'null'};
		}
	}

	function taxonomy(options, spec) {
		var extensions = gatherTaxonomyExtensions();
		spec = resolveTaxonomy(spec, extensions);
		var factory = extensions.taxonomies[spec.name];
		if (!factory) {
			return taxonomy(options, 'text/plain');
		}
		var taxonomyObj = factory(options, spec);
		if (extensions.taxonomyExtensions.hasOwnProperty(spec.name)) {
			var exts = extensions.taxonomyExtensions[spec.name];
			for (var prop in exts) {
				if (!exts.hasOwnProperty(prop)) {
					continue;
				}
				if (taxonomyObj.hasOwnProperty(prop)) {
					taxonomyObj['_' + prop] = taxonomyObj[prop];
				}
				taxonomyObj[prop] = exts[prop];
			}
		}
		taxonomyObj.name = spec.name;
		return taxonomyObj;
	}

	Extensions.add('com.foldingtext.editor.taxonomies', {
		name: 'null',
		configurations: [
			{ mime: 'text/plain', spec: 'null' }
		],
		taxonomy: function (spec) {
			return new Taxonomy(spec);
		}
	});

	exports.taxonomy = taxonomy;
});