//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions;

	Extensions.addInit(function (editor) {
		// http://www.hcs.harvard.edu/~jrus/site/system-bindings.html

		editor.addKeyMap({
			name: 'cocoa',

			'Enter' : 'insertNewline',
			'Cmd-Enter' : 'insertNewlineBelow',
			'Shift-Cmd-Enter' : 'insertNewlineAbove',
			'Alt-Enter' : 'insertNewlineIgnoringFieldEditor',

			'Tab' : 'insertTab',
			'Alt-Tab' : 'insertTabIgnoringFieldEditor',
			'Shift-Tab' : 'insertBacktab',

			'Backspace' : 'deleteBackward',
			'Shift-Backspace' : 'deleteBackward',
			'Ctrl-Backspace' : 'deleteBackwardByDecomposingPreviousCharacter',
			'Ctrl-Alt-Backspace' : 'deleteWordBackward',
			'Alt-Backspace' : 'deleteWordBackward',
			'Cmd-Backspace' : 'deleteToBeginningOfLine',
			'Delete' : 'deleteForward',
			'Shift-Delete' : 'deleteForward',
			'Alt-Delete' : 'deleteWordForward',

			'Esc' : 'cancelOperation',
			'Alt-Esc' : 'complete',

			'Up' : 'moveUp',
			'Shift-Up' : 'moveUpAndModifySelection',
			'Ctrl-Up' : 'scrollPageUp',
			'Cmd-Up' : 'moveToBeginningOfDocument',
			'Shift-Cmd-Up' : 'moveToBeginningOfDocumentAndModifySelection',
			'Alt-Up' : function (editor) {
				editor.performCommand('moveBackward');
				editor.performCommand('moveToBeginningOfParagraph');
			},
			'Shift-Alt-Up' : 'moveParagraphBackwardAndModifySelection',

			'Down' : 'moveDown',
			'Shift-Down' : 'moveDownAndModifySelection',
			'Ctrl-Down' : 'scrollPageDown',
			'Cmd-Down' : 'moveToEndOfDocument',
			'Shift-Cmd-Down' : 'moveToEndOfDocumentAndModifySelection',
			'Alt-Down' : function (editor) {
				editor.performCommand('moveForward');
				editor.performCommand('moveToEndOfParagraph');
			},
			'Shift-Alt-Down' : 'moveParagraphForwardAndModifySelection',

			'Left' : 'moveLeft',
			'Shift-Left' : 'moveLeftAndModifySelection',
			'Ctrl-Left' : 'moveToBeginningOfLine',
			'Shift-Ctrl-Left' : 'moveToBeginningOfLineAndModifySelection',
			'Cmd-Left' : 'moveToBeginningOfLine',
			'Shift-Cmd-Left' : 'moveToBeginningOfLineAndModifySelection',
			'Alt-Left' : 'moveWordLeft',
			'Shift-Alt-Left' : 'moveWordLeftAndModifySelection',

			'Right' : 'moveRight',
			'Shift-Right' : 'moveRightAndModifySelection',
			'Ctrl-Right' : 'moveToEndOfLine',
			'Shift-Ctrl-Right' : 'moveToEndOfLineAndModifySelection',
			'Cmd-Right' : 'moveToEndOfLine',
			'Shift-Cmd-Right' : 'moveToEndOfLineAndModifySelection',
			'Alt-Right' : 'moveWordRight',
			'Shift-Alt-Right' : 'moveWordRightAndModifySelection',

			'Home' : 'scrollToBeginningOfDocument',
			'Shift-Home' : 'moveToBeginningOfDocumentAndModifySelection',
			'End' : 'scrollToEndOfDocument',
			'Shift-End' : 'moveToEndOfDocumentAndModifySelection',

			'PageUp' : 'scrollPageUp',
			'Shift-PageUp' : 'pageUpAndModifySelection',
			'Alt-PageUp' : 'pageUp',
			'PageDown' : 'scrollPageDown',
			'Shift-PageDown' : 'pageDownAndModifySelection',
			'Alt-PageDown' : 'pageDown'
		});

		editor.addKeyMap({
			name: 'emacsy',

			'Cmd-.' : 'cancelOperation',
			'Ctrl-A' : 'moveToBeginningOfParagraph',
			'Shift-Ctrl-A' : 'moveToBeginningOfParagraphAndModifySelection',
			'Ctrl-B' : 'moveBackward',
			'Ctrl-D' : 'deleteForward',
			'Ctrl-E' : 'moveToEndOfParagraph',
			'Shift-Ctrl-E' : 'moveToEndOfParagraphAndModifySelection',
			'Ctrl-F' : 'moveForward',
			'Ctrl-H' : 'deleteBackward',
			'Ctrl-K' : 'deleteToEndOfParagraph',
			'Ctrl-L' : 'centerSelectionInVisibleArea',
			'Ctrl-N' : 'moveDown',
			'Ctrl-O' : function (editor) {
				editor.performCommand('insertNewlineIgnoringFieldEditor');
				editor.performCommand('moveBackward');
			},
			'Ctrl-P' : 'moveUp',
			'Ctrl-T' : 'transpose',
			'Ctrl-V' : 'pageDown',
			'Ctrl-Y' : 'yank',

			'F5' : 'complete'
		});
	});
});