//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions,
		codemirrorCSS = require('codemirror').CodeMirrorCSS,
		less = require('components/less.js/dist/less-1.7.0'),
		baseLESS = require('components/text/text!apps/base.less'),
		queryParameter = require('ft/util/queryparameter').QueryParameter,
		evaledVariables,
		styleElement;

	function loadCss(cssText, callback) {
		if (!styleElement) {
			styleElement = document.createElement('style');
			styleElement.type = 'text/css';
			styleElement.onload = function () {
				callback();
			};
		}
		styleElement.innerHTML = cssText;
		if (!styleElement.parent) {
			document.head.appendChild(styleElement);
		}
	}

	function readCSS() {
		if (styleElement) {
			return styleElement.innerText;
		}
		return '';
	}

	function readStyleVariable(name, defaultValue) {
		var variable = evaledVariables ? evaledVariables[name] : null,
			value = variable ? variable.value : null,
			/*jshint evil:true */
			evaluatedValue = value ? value['eval']() : null,
			/*jshint evil:false */
			result = evaluatedValue ? evaluatedValue.value : null;
		if (result === null) {
			return defaultValue;
		} else {
			return result;
		}
	}

	function readNumberStyleVariable(name, defaultValue) {
		var result = readStyleVariable(name, defaultValue);
		if (typeof(result) === 'string') {
			result = parseInt(result, 10);
		}
		return result;
	}

	function themes() {
		var results = [];
		Extensions.processExtensions('com.foldingtext.themes', function (each) {

		});
		return results;
	}

	function loadLESS(taxonomyStyles, appStyles, completionHandler, ignorePluginStyles, ignoreUserStyles) {
		var parser = new less.Parser({
				env: 'production',
				async: true,
				fileAsync: true,
				relativeUrls: true,
				filename: 'less.template'
			}),
			pluginStyles = ignorePluginStyles ? '' : queryParameter('pluginStyles', ''),
			userStyles = ignoreUserStyles ? '' : queryParameter('userStyles', ''),
			systemSelectionColor = queryParameter('systemSelectionColor', '#a7caff'),
			lessTemplate = baseLESS;

		if (pluginStyles) {
			pluginStyles = '@import (less) "' + pluginStyles.split(':').join('";\n@import (less) "') + '";';
		}

		if (userStyles) {
			userStyles = '@import (less) "' + userStyles.split(':').join('";\n@import (less) "') + '";';
		}

		var extensionStyles = [];
		Extensions.processExtensions('com.foldingtext.editor.styles', function (each) {
			extensionStyles.push(each);
		});

		lessTemplate = lessTemplate.replace('CODEMIRROR_STYLES', codemirrorCSS);
		lessTemplate = lessTemplate.replace('SYSTEM_SELECTION_COLOR', systemSelectionColor);
		lessTemplate = lessTemplate.replace('TAXONOMY_STYLES', taxonomyStyles);
		lessTemplate = lessTemplate.replace('EXTENSION_POINT_STYLES', extensionStyles.join('\n\n'));
		lessTemplate = lessTemplate.replace('APP_STYLES', appStyles);
		lessTemplate = lessTemplate.replace('PLUGIN_STYLES', pluginStyles);
		lessTemplate = lessTemplate.replace('USER_STYLES', userStyles);

		parser.parse(lessTemplate, function (err, tree) {
			if (err) {
				console.error(err);
				if (ignorePluginStyles) {
					console.log('Less error, will ignore user sytles next');
					ignoreUserStyles = true;
				} else {
					console.log('Less error, will ignore plugin sytles next');
					ignorePluginStyles = true;
				}
				loadLESS(taxonomyStyles, appStyles, completionHandler, ignorePluginStyles, ignoreUserStyles);
			} else {
				var options = { plugins : [{
						isPreVisitor : true,
						run : function (evaledRoot) {
							evaledVariables = evaledRoot.variables();
						}
					}]},
					variables = {};
				loadCss(tree.toCSS(options, variables), completionHandler);
			}
		});
	}

	exports.themes = themes;
	exports.loadLESS = loadLESS;
	exports.readCSS = readCSS;
	exports.readStyleVariable = readStyleVariable;
	exports.readNumberStyleVariable = readNumberStyleVariable;
});