define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions,
		Tree = require('ft/core/tree').Tree;

	/*jshint validthis:true */

	function cut(clipboard) {
		this.copy(clipboard);
		this.replaceSelections('', 'end', 'cut');
	}

	function copy(clipboard) {
		var selectedRanges = this.selectedRanges(),
			selectedTextAndFolds = [],
			selectedText = [];

		for (var i = 0; i < selectedRanges.length; i++) {
			var eachRange = selectedRanges[i],
				eachText = eachRange.textInRange(),
				eachFolds = this.foldsInRange(eachRange),
				eachFoldsJSON = this.foldsToJSON(eachFolds);

			this.normalizeFoldsJSON(eachFoldsJSON, eachRange.startLineCh());

			selectedText.push(eachText);
			selectedTextAndFolds.push({
				text : eachText,
				folds : eachFoldsJSON
			});
		}

		clipboard.setData('text/plain', selectedText.join('\n'));
		clipboard.setData('com.foldingtext.FoldingText', JSON.stringify(selectedTextAndFolds));
	}

	function paste(clipboard) {
		var selectedRanges = this.selectedRanges(),
			pastedText = clipboard.getData('text/plain'),
			pastedTextAndFoldsJSON = clipboard.getData('com.foldingtext.FoldingText');

		if (pastedTextAndFoldsJSON) {
			var pastedTextAndFolds = JSON.parse(pastedTextAndFoldsJSON),
				pastedTextAndFoldsLength = pastedTextAndFolds.length,
				foldOffset = { line: 0, ch: 0},
				each, eachFolds, eachTextLines,
				i;

			if (pastedTextAndFolds.length !== selectedRanges.length) {
				var consolidatedPastedText = [],
					consolidatedPastedFolds = [];

				for (i = 0; i < pastedTextAndFoldsLength; i++) {
					each = pastedTextAndFolds[i];
					eachTextLines = Tree.stringByNormalizingLineEndings(each.text).split('\n');
					eachFolds = each.folds;	
					consolidatedPastedText = consolidatedPastedText.concat(eachTextLines);
					this.perpareNormalizedFoldsJSONForDestination(eachFolds, foldOffset);
					consolidatedPastedFolds = consolidatedPastedFolds.concat(eachFolds);
					foldOffset.line += eachTextLines.length;
				}

				pastedTextAndFolds = [];

				for (i = 0; i < selectedRanges.length; i++) {
					pastedTextAndFolds.push({
						text: consolidatedPastedText.join('\n'),
						folds: JSON.parse(JSON.stringify(consolidatedPastedFolds))
					});
				}
			}

			foldOffset.line = 0;
			foldOffset.ch = 0;

			var pastedFolds = [],
				pastedTexts = [];

			for (i = 0; i < pastedTextAndFolds.length; i++) {
				var destinationRange = selectedRanges[i],
					destinationLineCh = destinationRange.startLineCh(),
					eachTextAndFolds = pastedTextAndFolds[i],
					textLines = Tree.stringByNormalizingLineEndings(eachTextAndFolds.text).split('\n'),
					foldsJSON = eachTextAndFolds.folds;

				
				destinationLineCh.line += foldOffset.line;
				//destinationLineCh.ch += foldOffset.ch;

				foldsJSON = this.perpareNormalizedFoldsJSONForDestination(foldsJSON, destinationLineCh);
				
				pastedTexts.push(eachTextAndFolds.text);
				pastedFolds = pastedFolds.concat(foldsJSON);						

				foldOffset.line += textLines.length - 1;
			}

			this.replaceSelections(pastedTexts, 'end', 'paste');
			this.restoreJSONFolds(pastedFolds);
		} else if (pastedText !== null) {
			this.replaceSelections(Tree.stringByNormalizingLineEndings(pastedText).split('\n'), 'end', 'paste');
		}
	}

	Extensions.add('com.foldingtext.editor.methods', {
		cut: cut,
		copy: copy,
		paste: paste
	});

	Extensions.addInit(function (editor) {
		var cm = editor.cm();

		cm.on('cut', function (cm, e) {
			e.preventDefault();
			editor.cut(e.clipboardData);
		});

		cm.on('copy', function (cm, e) {
			e.preventDefault();
			editor.copy(e.clipboardData);
		});

		cm.on('paste', function (cm, e) {
			e.preventDefault();
			editor.paste(e.clipboardData);
		});
	});
});