define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions;

	Extensions.addInit(function (editor) {
		var dropIndicatorDiv = document.createElement('div'),
			draggedTextMarkers,
			cm = editor.cm();

		dropIndicatorDiv.className = 'CodeMirror-dropIndicator';

		//
		// Dragging Source
		//

		cm.on('dragstart', function (cm, e) {
			e.dataTransfer.effectAllowed = 'copyMove';
			draggedTextMarkers = [];
			cm.operation(function () {
				cm.listSelections().forEach(function (each) {
					draggedTextMarkers.push(cm.markText(each.anchor, each.head));
				});
			});
		});

		cm.on('dragend', function (cm, e) {
			cm.operation(function () {
				draggedTextMarkers.forEach(function (each) {
					if (e.dataTransfer.dropEffect === 'move') {
						var range = each.find();
						if (range) {
							cm.replaceRange('', range.from, range.to, '+paste');
						}
					}
					each.clear();
				});
				draggedTextMarkers = undefined;				
			});
		});

		//
		// Drop Target
		//

		function showDragIndicator() {
			if (!dropIndicatorDiv.parentNode) {
				cm.display.lineSpace.appendChild(dropIndicatorDiv);
			}
		}

		function removeDragIndicator() {
			if (dropIndicatorDiv.parentNode) {
				dropIndicatorDiv.parentNode.removeChild(dropIndicatorDiv);
			}
		}

		cm.on('dragenter', function (cm, e) {
			showDragIndicator(e);
		});

		cm.on('dragover', function (cm, e) {
			var lineCh = editor.targetsAt(e.clientX, e.clientY).lineCh,
				coords = editor.cursorCoords(lineCh, 'div');
			if (cm.state.draggingText && cm.doc.sel.contains(lineCh) > -1) {
				e.dataTransfer.dropEffect = 'none';
				removeDragIndicator(e);
			} else {
				dropIndicatorDiv.style.left = coords.left + 'px';
				dropIndicatorDiv.style.top = coords.top + 'px';
				dropIndicatorDiv.style.height = Math.max(0, coords.bottom - coords.top) * cm.options.cursorHeight + 'px';
				showDragIndicator(e);
			}

		});

		cm.on('dragleave', function (cm, e) {
			removeDragIndicator(e);
		});

		cm.on('drop', function (cm, e) {
			removeDragIndicator(e);
		});
	});
});



