define(function (require, exports, module) {
	'use strict';

	require('ft/extensions/folding');
	require('ft/extensions/nodepath');

	var CodeMirror = require('codemirror').CodeMirror,
		Extensions = require('ft/core/extensions').Extensions,
		NodePath = require('ft/core/nodepath').NodePath,
		Range = require('ft/core/range').Range;

	/*jshint validthis:true */

	function focusRange(range) {
		var editor = this,
			tree = editor.tree(),
			start = range.location(),
			end = start + range.length(),
			preRange = tree.createRangeFromLocation(0, start),
			postRange = tree.createRangeFromLocation(end, tree.textLength() - end);

		editor.operation(function () {
			editor.removeFolds(editor.folds());
			editor.createFoldInRange(preRange, false);
			editor.createFoldInRange(postRange, false);
		});
	}

	function focusNode(node) {
		if (node.firstChild || node.type() === 'heading') {
			this.setNodePath(NodePath.calculateMinNodePath(node, true));
			this.scrollToTop();
		} else {
			node = node.parent;
			if (node.isRoot()) {
				this.setNodePath(null);
			} else {
				this.setNodePath(NodePath.calculateMinNodePath(node, true));
			}
		}
	}

	function focusIn(editor) {
		var range = editor.selectedRange().rangeByAdjustingToEffectedNodesInEditor(editor);
		if (range.startNode === range.endNode) {
			editor.focusNode(range.startNode);
		} else {
			range = range.rangeByExtendingToParagraph();
			if (range.location() > 0) {
				range = range.rangeByOffsetting(-1, 1);
			}
			editor.focusRange(range);
		}
	}

	function focusOut(editor) {
		var nodePath = editor.nodePath();
		if (nodePath.isShowAllPath()) {
			var viewport = editor.viewport();
			editor.removeFolds(editor.folds());
			editor.scrollToLine(viewport.firstVisibleLine, viewport.firstVisibleLineOffset);
		} else {
			editor.setNodePath(NodePath.SHOW_ALL_PATH);
		}
	}

	Extensions.addCommand({
		name: 'focusIn',
		performCommand: focusIn
	});

	Extensions.addCommand({
		name: 'focusOut',
		validate: function (editor) {
			return !editor.nodePath().isShowAllPath() || editor.folds().length > 0;
		},
		performCommand: focusOut
	});

	Extensions.add('com.foldingtext.editor.willEndOperation', function (editor) {
		window.setTimeout(function () {
			if (editor._editorTopFold) {
				if (!editor.nodePath().isShowAllPath() || editor.folds().length > 0) {
					editor._editorTopFold.style.display = '';
				} else {
					editor._editorTopFold.style.display = 'none';
				}
			}
		}, 0);
	});

	Extensions.addInit(function (editor) {
		var wrapperElement = editor.wrapperElement(),
			topFoldWidget = document.createElement('div');

		topFoldWidget.className = 'cm-editorTopFold';
		topFoldWidget.style.display = 'none';
		topFoldWidget.title = 'Focus Out';

		editor._editorTopFold = topFoldWidget;

		CodeMirror.on(topFoldWidget, 'mousedown', function (e) {
			editor.performCommand('focusOut');
			e.preventDefault();
		});

		wrapperElement.appendChild(topFoldWidget);
	});

	Extensions.add('com.foldingtext.editor.methods', {
		focusRange: focusRange,
		focusNode: focusNode
	});
});