define(function (require, exports, module) {
	'use strict';

	var CodeMirror = require('codemirror').CodeMirror,
		Extensions = require('ft/core/extensions').Extensions,
		theme = require('ft/editor/theme');

	require('ft/extensions/measuretext');

	Extensions.add('com.foldingtext.editor.options', {
		name: 'padBottom',
		defaultValue: true,
		onChange: function (editor, oldValue, newValue) {
			if (oldValue === newValue) {
				return;
			}
		
			readjustBottomPadding(editor);
			editor.invalidateRestorableState();
		}
	});

	Extensions.add('com.foldingtext.editor.refresh', function (editor) {
		readjustBottomPadding(editor);
		readjustWrapAmount(editor);
	});

	Extensions.add('com.foldingtext.editor.options', {
		name: 'wrapToColumn',
		defaultValue: 66,
		onChange: function (editor, oldValue, newValue) {
			if (oldValue === newValue) {
				return;
			}

			readjustWrapAmount(editor);
			readjustBottomPadding(editor);
			editor.invalidateRestorableState();
		}
	});

	Extensions.addCommand({
		name: 'wrapToColumn',
		validate: function (editor, options) {
			options.interfaceItemState = editor.option('wrapToColumn') === options.interfaceItemTag;
			return true;
		},
		performCommand: function (editor, options) {
			editor.setOption('wrapToColumn', options.interfaceItemTag);
		}
	});

	Extensions.add('com.foldingtext.editor.methods', {
		refresh: function () {
			this.clearMeasureTextCache();
			this.cm().refresh();
		},

		wrapperElement: function () {
			return this.cm().getWrapperElement();
		},

		scrollerElement: function () {
			return this.cm().getScrollerElement();
		},

		linesElement: function () {
			return this.cm().display.mover.firstElementChild;
		},

		textWrapWidth: function () {
			return this.option('wrapToColumn') * this.defaultCharWidth();
		}
	});

	function readjustWrapAmount(editor) {
		var linesElement = editor.linesElement(),
			textWrapWidth = editor.textWrapWidth(),
			preStyling = editor.defaultPREStyling(),
			pageMarginWidth = preStyling.marginLeft + preStyling.marginRight,
			maxWidth;
			
		if (textWrapWidth === 0) {
			maxWidth = 'none';
		} else {
			maxWidth = Math.round(textWrapWidth + pageMarginWidth) + 'px';
		}

		if (editor._lastSetMaxWidth !== maxWidth) {
			editor._lastSetMaxWidth = maxWidth;
			linesElement.style.maxWidth = maxWidth;
			editor.refresh();
		}
	}

	function readjustBottomPadding(editor) {
		var linesElement = editor.linesElement(),
			padBottom = editor.option('padBottom'),
			newBottomPadding = 0,
			newMinHeight = 0;
			
		if (padBottom) {
			var viewHeight = editor.wrapperElement().clientHeight,
				existingTopPadding = parseInt(window.getComputedStyle(linesElement).paddingTop) || 0,
				paddingPercent = 0.5;

			newBottomPadding = Math.floor(viewHeight * paddingPercent);
			newMinHeight = Math.floor(viewHeight * (1 - paddingPercent));
			newMinHeight -= existingTopPadding;
			if (newMinHeight < 0) {
				newMinHeight = 0;
			}
		}

		if (editor._lastSetPaddingBottom !== newBottomPadding || editor._lastSetMinHeight !== newMinHeight) {
			editor._lastSetMinHeight = newMinHeight;
			editor._lastSetPaddingBottom = newBottomPadding;
			linesElement.style['min-height'] = newMinHeight + 'px';
			linesElement.style.paddingBottom = newBottomPadding + 'px';
			editor.cm().setSize();
		}
	}
});