define(function (require, exports, module) {
	'use strict';

	require('searchcursor');
	require('ft/extensions/panel');

	var Extensions = require('ft/core/extensions').Extensions;

	function getSearchCursor(editor, query, pos) {
		return editor.cm().getSearchCursor(query, pos, typeof query === 'string' && query === query.toLowerCase());
	}

	function updateLeap(editor, from, pattern, backward) {
		var searchCursor = getSearchCursor(editor, pattern, from),
			cm = editor.cm();

		editor._leapFrom = from;

		if (!searchCursor.find(backward)) {
			searchCursor = getSearchCursor(editor, pattern, backward ? CodeMirror.Pos(cm.lastLine()) : CodeMirror.Pos(cm.firstLine(), 0));
			if (!searchCursor.find(backward)) {
				cm.setCursor(editor._leapFrom);
				return;
			}
		}

		cm.setSelection(searchCursor.from(), searchCursor.to());
	}

	function showLeapUI(editor, from) {
		var panel = editor.panel();
		panel.show();
		panel.removeField('leap');
		panel.addField({
			id: 'leap',
			label: 'Leap:',
			value: '',
			onChange: function (oldValue, newValue) {
				updateLeap(editor, from, newValue);
			}
		});

		panel.focus();
		// when panel looses focus, then close it.
		// or when main editor gets focus, then close panel.
	}

	function leapRight(editor) {
		showLeapUI(editor, editor.cm().getCursor());
	}

	function leapRightAgain(editor) {
		updateLeap(editor, editor.cm().getCursor(), editor.panel().fieldValue('leap'), false);
		return true;
	}

	function leapLeft(editor) {
	}

	function leapLeftAgain(editor) {
	}

	Extensions.addCommand({
		name: 'leapRight',
		performCommand: leapRight
	});

	Extensions.addCommand({
		name: 'leapRightAgain',
		performCommand: leapRightAgain
	});

	Extensions.addCommand({
		name: 'leapLeft',
		performCommand: leapLeft
	});

	Extensions.addCommand({
		name: 'leapLeftAgain',
		performCommand: leapLeftAgain
	});
});