define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions,
		Environment = require('ft/util/environment'),
		Delay = require('ft/util/delay').Delay,
		Undo = require('ft/extensions/undo');

	function changed(cm) {
		cm.autosaveDelay.clear();
		cm.autosaveDelay.set(1000, function () {
			saveIfNeeded(cm);
		});
	}

	function saveIfNeeded(cm, saveState) {
		if (!cm.isClean()) {
			window.localStorage.setItem('text', cm.getValue());
			cm.markClean();
			if (window._gaq) {
				window._gaq.push(['_trackEvent', 'localStorage', 'saveText']);
			}
		}

		if (saveState) {
			window.localStorage.setItem('state', cm.getOption('ftEditor').restorableStateJSON());
			if (window._gaq) {
				window._gaq.push(['_trackEvent', 'localStorage', 'saveState']);
			}
		}
	}

	function hasLocalStorage() {
		try {
			return 'localStorage' in window && window.localStorage !== null;
		} catch (e) {
			return false;
		}
	}

	function startUsingLocalStorage(editor) {
		window.onbeforeunload = function () {
			saveIfNeeded(editor.cm(), true);
			return null;
		};
		var text = window.localStorage.getItem('text'),
			state = window.localStorage.getItem('state'),
			cm = editor.cm();

		if (text) {
			editor.setTextContent(text);
			editor.performCommand('clearUndoHistory');
		}

		if (state) {
			editor.setRestorableStateJSON(state);
		}

		cm.autosaveDelay = new Delay();
		cm.on('change', changed);
	}

	function stopUsingLocalStorage(editor) {
		var cm = editor.cm();
		if (cm.autosaveDelay) {
			cm.autosaveDelay.clear();
		}
		cm.off('change', changed);
	}

	Extensions.addInit(function (editor) {
		if (!Environment.disableLocalStorage) {
			if (hasLocalStorage()) {
				window.setTimeout(function () {
					startUsingLocalStorage(editor);
				}, 0);
			} else {
				window.onbeforeunload = function () {
					return 'Local storage is not supported in your browser. You must copy and paste you text to some other place to save it.';
				};
			}
		}
	});
});