define(function (require, exports, module) {
	'use strict';

	require('ft/util/array');

	var Extensions = require('ft/core/extensions').Extensions;

	var spaceStrs = [''];
	function spaceStr(n) {
		while (spaceStrs.length <= n) {
			spaceStrs.push(spaceStrs.lastObject() + ' ');
		}
		return spaceStrs[n];
	}

	var cacheKeySegmentCache = {},
		cacheKeySegmentCacheNext = 0;

	function cantorPairing(x, y) {
		return ((x + y) * (x + y + 1)) / 2 + y;
	}

	function shortenCacheKeySegment(cacheKeySegment) {
		var result = cacheKeySegmentCache[cacheKeySegment];
		if (!result) {
			result = cacheKeySegmentCacheNext++;
			cacheKeySegmentCache[cacheKeySegment] = result;
		}
		return result;
	}

	function cacheKeyForElement(element, includeTextContent) {
		var attributes = element.attributes,
			result = 0,
			attr;

		for (var j = 0; j < attributes.length; j++) {
			attr = attributes[j];
			result = cantorPairing(result, shortenCacheKeySegment(attr.name + attr.value));
		}

		if (includeTextContent) {
			result += element.textContent;
		}

		return result;
	}

	var measureElement = document.createElement('pre'),
		measureSpan = document.createElement('span');

	measureSpan.style.whiteSpace = 'pre-wrap';
	measureElement.appendChild(measureSpan);

	Extensions.add('com.foldingtext.editor.methods', {

		measureTextCache: function (argument) {
			var cache = this._measureTextCache;
			if (!cache) {
				cache = {};
				this._measureTextCache = cache;
			}
			return cache;
		},

		defaultCharWidth: function () {
			return this.cm().defaultCharWidth();
		},

		defaultSpaceWidth: function () {
			return this.cm().defaultSpaceWidth();
		},

		defaultPREStyling: function () {
			var measureTextCache = this.measureTextCache(),
				defaultPREStyling = measureTextCache.defaultPREStyling;

			if (!defaultPREStyling) {
				var cm = this.cm(),
					measureDiv = cm.display.measure;
				
				defaultPREStyling = {};
				measureTextCache.defaultPREStyling = defaultPREStyling;
				measureDiv.appendChild(measureElement);
				var style = window.getComputedStyle(measureElement);
				defaultPREStyling.marginTop = style.marginTop ? parseFloat(style.marginTop) : 0;
				defaultPREStyling.marginBottom = style.marginBottom ? parseFloat(style.marginBottom) : 0;
				defaultPREStyling.marginLeft = style.marginLeft ? parseFloat(style.marginLeft) : 0;
				defaultPREStyling.marginRight = style.marginRight ? parseFloat(style.marginRight) : 0;
				measureDiv.removeChild(measureElement);				
			}

			return defaultPREStyling;
		},

		tabWidth: function () {
			return this.cm().getOption('tabSize') * this.defaultSpaceWidth();
		},

		measureNodeText: function (node, location, length, cmBuilder, cmMeasure) {
			if (length === 0) {
				return { top: 0, bottom: 0, left: 0, right: 0, width: 0, height: 0 };
			}

			var cacheKey = shortenCacheKeySegment(cmBuilder.wrapClass),
				measureTextCache = this.measureTextCache(),
				startNode,
				startOffset,
				endNode,
				endOffset,
				end = location + length,
				map = cmMeasure.map,
				i = 0;

			cacheKey += cacheKeyForElement(cmBuilder.pre, false);

			// Make contains only text nodes and widgets
			while (i < map.length && (startNode === undefined || endNode === undefined)) {
				var s = map[i],
					e = map[i + 1],
					n = map[i + 2];

				if (!startNode && location <= e) {
					startNode = n;
					startOffset = location - s;
				}

				if (!endNode && end <= e) {
					endNode = n;
					endOffset = (end - s);
				}

				if (startNode) {
					// we are in run of spans to measure. So make each span in
					// that run part of the cache key
					if (n.nodeType === document.TEXT_NODE) {
						var spanParent = n.parentNode;
						if (spanParent.className) {
							cacheKey += cacheKeyForElement(spanParent, true);
						} else {
							cacheKey += n.textContent;
						}
					} else {
						// widget
						cacheKey += shortenCacheKeySegment(n.innerHTML);
					}
				}

				i+=3;
			}

			var rect = measureTextCache[cacheKey];

			if (!rect) {
				var cm = this.cm(),
					measureDiv = cm.display.measure,
					range = document.createRange();

				if (startNode.nodeType === document.TEXT_NODE) {
					range.setStart(startNode, startOffset);
				} else {
					// widget or tab
					if (startOffset > 0) {
						startOffset = 1;
					}
					range.setStart(startNode, startOffset);
				}

				if (endNode.nodeType === document.TEXT_NODE) {
					range.setEnd(endNode, endOffset);
				} else {
					if (endOffset > 0) {
						endOffset = 1;
					}
					range.setEnd(endNode, endOffset);					
				}

				measureDiv.appendChild(cmBuilder.pre);
				rect = range.getBoundingClientRect();
				measureDiv.removeChild(cmBuilder.pre);

				measureTextCache[cacheKey] = rect;				
			}

			return rect;
		},

		clearMeasureTextCache: function () {
			this._measureTextCache = {};
		}
	});
});