define(function (require, exports, module) {
	'use strict';

	require('ft/extensions/nodepath');

	var CodeMirror = require('codemirror').CodeMirror,
		Extensions = require('ft/core/extensions').Extensions;

	function mouseDown(editor, e) {
		var attributes = editor.targetsAt(e.clientX, e.clientY).attributes;
		if (attributes.linklabel || attributes.linktarget || attributes.tagname || attributes.tagvalue) {
			return true;
		}
		return false;
	}

	function mouseUp(editor, e) {
		if (e.button === 2 || (e.button === 0 && e.ctrlKey)) {
			return false;
		}

		var targets = editor.targetsAt(e.clientX, e.clientY),
			attributes = targets.attributes,
			lineCh = targets.lineCh,
			node = targets.node,
			target;

		function findLinkDefinitionFromTargetLabel(targetLabel) {
			while (node) {
				var type = node.type();
				if (type === 'linkdefinition' || type === 'footnotelinkdefinition') {
					if (node.lineAttributedString().findAttributeValue('linkid') === targetLabel) {
						return node;
					}
				}
				node = node.nextLineNode();
			}
			return null;
		}

		function findLinkFromTargetLabel(targetLabel) {
			var node = findLinkDefinitionFromTargetLabel(targetLabel);
			if (node) {
				var t = node.lineAttributedString().findAttributeValue('linktarget');
				if (t) {
					return t;
				} else {
					node = node.nextLineNode();
					while (node) {
						type = node.type();
						if (type === 'linkdefinitionurl') {
							return node.lineAttributedString().findAttributeValue('linktarget');
						} else if (type === 'empty') {
							continue;
						} else {
							return null;
						}
						node = node.nextLineNode();
					}
				}				
			}
			return null;
		}

		var inBackground = false;
		if (CodeMirror.mac) {
			inBackground = e.metaKey;
		} else {
			inBackground = e.ctrlKey;
		}

		if (attributes.linktarget) {
			target = attributes.linktarget;
			attributes = node.lineAttributesAtIndex(lineCh.ch);
			if (attributes.referencelink) {
				editor.openLink(findLinkFromTargetLabel(target), inBackground);
			} else {
				editor.openLink(target, inBackground);
			}
			return true;
		} else if (attributes.linklabel) {
			var linkLabel = attributes.linklabel;
			target = node.lineAttributedString().findAttributeValue('linktarget', lineCh.ch);
			attributes = node.lineAttributesAtIndex(lineCh.ch);
			if (attributes.referencelink) {
				if (!target || !target.trim()) {
					target = linkLabel;
				}
				editor.openLink(findLinkFromTargetLabel(target), inBackground);
				return true;
			} else if (attributes.footnotelink) {
				target = linkLabel;
				var linkDefinition = findLinkDefinitionFromTargetLabel(target);
				if (linkDefinition) {
					editor.scrollRangeToVisible(editor.tree().createRangeFromNodes(linkDefinition, 0));
				}
				return true;
			} else if (attributes.inlinelink) {
				editor.openLink(target, inBackground);
				return true;
			}
		} else if (attributes.tagname) {
			editor.setNodePathFilter('@' + attributes.tagname);
			return true;
		} else if (attributes.tagvalue) {
			var tagvalue = attributes.tagvalue,
				location = targets.lineCh.ch,
				effectiveRange = {};

			node = targets.node;

			while (location >= 0) {
				attributes = node.lineAttributesAtIndex(location, effectiveRange);
				if (attributes.tagname) {
					editor.setNodePathFilter('@' + attributes.tagname + ' = "' + tagvalue + '"');
					return true;
				} else {
					location = effectiveRange.location - 1;
				}
			}
			return true;
		}
		return false;
	}

	Extensions.add('com.foldingtext.editor.mouse', {
		mouseDown: mouseDown,
		mouseUp: mouseUp
	});
});