define(function (require, exports, module) {
	'use strict';

	require('ft/extensions/folding');
	require('ft/extensions/foldingnodes');

	var Extensions = require('ft/core/extensions').Extensions,
		Selection = require('ft/extensions/selection');

	function moveParagraphs(editor, direction) {
		var selectedRange = editor.selectedRange(),
			effectedRange = selectedRange.rangeByAdjustingToEffectedNodesInEditor(editor),
			nodes = effectedRange.nodesInRange(),
			length = nodes.length,
			tree = editor.tree(),
			cm = editor.cm(),
			i;

		if (direction === 'right') {
			Selection.performNodeOperationWhileMaintainingEditorStateInRange(editor, selectedRange, effectedRange, function() {
				tree.beginUpdates();
				for (i = 0; i < length; i++) {
					nodes[i].replaceLineInRange('\t', 0, 0);
				}
				tree.endUpdates();
			});
			return;
		}

		if (direction === 'left') {
			Selection.performNodeOperationWhileMaintainingEditorStateInRange(editor, selectedRange, effectedRange, function() {
				tree.beginUpdates();
				for (i = 0; i < length; i++) {
					var each = nodes[i];
					if (each.line()[0] === '\t') {
						nodes[i].replaceLineInRange('', 0, 1);
					}
				}
				tree.endUpdates();
			});
			return;
		}

		var insertBefore;

		if (direction === 'up') {
			insertBefore = nodes[0].previousLineNode();

			if (!insertBefore) {
				return;
			}

			if (editor.nodeIsHiddenInFold(insertBefore)) {
				while (insertBefore && editor.nodeIsHiddenInFold(insertBefore)) {
					insertBefore = insertBefore.previousLineNode();
				}

				if (insertBefore && !editor.isCollapsed(insertBefore)) {
					insertBefore = insertBefore.nextLineNode();
				}
			}

			if (!insertBefore) {
				insertBefore = tree.firstLineNode();
			}	
		}
		
		if (direction === 'down') {
			var nextLineNode = nodes[nodes.length - 1].nextLineNode();

			if (!nextLineNode) {
				return;
			}

			insertBefore = nextLineNode.nextLineNode();

			if (editor.isCollapsed(nextLineNode) || editor.nodeIsHiddenInFold(nextLineNode)) {
				while (insertBefore && editor.nodeIsHiddenInFold(insertBefore)) {
					insertBefore = insertBefore.nextLineNode();
				}
			}
		}

		Selection.performNodeOperationWhileMaintainingEditorStateInRange(editor, selectedRange, effectedRange, function() {
			tree.beginUpdates();
			tree.removeNodes(nodes);
			tree.insertNodesBefore(nodes, insertBefore);
			tree.endUpdates();
		});
	}

	function deleteParagraphs(editor) {
		var tree = editor.tree(),
			selectedRange = editor.selectedRange(),
			range = selectedRange.rangeByAdjustingToEffectedNodesInEditor(editor),
			nodes = range.nodesInRange();

		if (!selectedRange.isCollapsed()) {
			editor.replaceSelection('');
			return;
		}

		if (nodes.length === tree.nodeCount()) {
			tree.setText('');
		} else {
			tree.removeNodes(range.nodesInRange());
		}
	}

	Extensions.addCommand({
		name: 'moveParagraphsUp',
		performCommand: function (editor) {
			moveParagraphs(editor, 'up');
		}
	});

	Extensions.addCommand({
		name: 'moveParagraphsDown',
		performCommand: function (editor) {
			moveParagraphs(editor, 'down');
		}
	});

	Extensions.addCommand({
		name: 'moveParagraphsRight',
		performCommand: function (editor) {
			moveParagraphs(editor, 'right');
		}
	});

	Extensions.addCommand({
		name: 'moveParagraphsLeft',
		performCommand: function (editor) {
			moveParagraphs(editor, 'left');
		}
	});

	Extensions.addCommand({
		name: 'deleteParagraphs',
		performCommand: deleteParagraphs
	});	
});