define(function (require, exports, module) {
	'use strict';

	var Folding = require('ft/extensions/folding'),
		Extensions = require('ft/core/extensions').Extensions,
		FoldingNodes = require('ft/extensions/foldingnodes'),
		NodePath = require('ft/core/nodepath').NodePath,
		NodeSet = require('ft/core/nodeset').NodeSet,
		Node = require('ft/core/node').Node,
		dom = require('ft/util/dom');

	/*jshint validthis:true */

	function nodePath() {
		if (!this._nodePath) {
			this._nodePath = new NodePath(NodePath.SHOW_ALL_PATH, this.tree().taxonomy.types());
		}
		return this._nodePath;
	}

	function nodePathMatches() {
		if (!this._nodePathMatches) {
			this._nodePathMatches = this.tree().evaluateNodePath(this._nodePath);
		}
		return this._nodePathMatches;
	}

	function nodePathMatchesNodeSet() {
		if (!this._nodePathMatchesNodeSet) {
			this._nodePathMatchesNodeSet = NodeSet.setFromArray(this.nodePathMatches());
		}
		return this._nodePathMatchesNodeSet;
	}

	function nodePathEditedNodeSet() {
		if (!this._nodePathEditedNodeSet) {
			this._nodePathEditedNodeSet = new NodeSet();
		}
		return this._nodePathEditedNodeSet;
	}

	function addFoldsToShowMatchesAndAncestors(editor) {
		var tree = editor.tree(),
			firstLineNode = tree.firstLineNode(),
			lastLineNode = tree.lastLineNode(),
			matches = editor.nodePathMatches().slice(0),
			eachMatch = matches.shift(),
			each = tree.firstLineNode(),
			foldStart, foldEnd, foldRange,
			folds = [];

		//var matchesLength = matches.length;
		//for (var j = 0; j < matchesLength; j++) {
		//	editor.setNeedsRender(matches[j]);
		//}

		if (eachMatch && eachMatch.isRoot()) {
			eachMatch = matches.shift();
		}

		while (each) {
			if (each === eachMatch) {
				if (foldStart) {
					foldEnd = each.previousLineNode();
					foldRange = tree.createRangeFromNodes(foldStart, 0, foldEnd, foldEnd.line().length);
					folds.push(editor.createFoldInRange(foldRange, false));
					foldStart = null;
				}
				eachMatch = matches.shift();
			} else if (!foldStart) {
				foldStart = each;
			}
			each = each.nextLineNode();
		}

		if (foldStart) {
			foldEnd = tree.lastLineNode();
			foldRange = tree.createRangeFromNodes(foldStart, 0, foldEnd, foldEnd.line().length);
			folds.push(editor.createFoldInRange(foldRange, false));
		}

		folds.forEach(function (eachFold) {
			if (eachFold) {
				var eachRange = eachFold.range();

				if (!editor._nodePathTopBoundaryFold) {
					if (eachRange.startNode === firstLineNode && eachRange.startOffset === 0) {
						editor._nodePathTopBoundaryFold = eachFold;
						eachFold.setIsBoundary(true);
					}
				}

				if (!editor._nodePathBottomBoundaryFold) {
					if (eachRange.endNode === lastLineNode && eachRange.endOffset === eachRange.endNode.line().length) {
						editor._nodePathBottomBoundaryFold = eachFold;
						eachFold.setIsBoundary(true);
					}
				}				
			}
		});
	}

	function addFoldsToShowMatchesAndAncestorsAndPreserveTreeStructure(editor) {
		var tree = editor.tree(),
			matches = editor.nodePathMatches(),
			matchesAncestors = Node.ancestorsForNodes(matches),
			stack = [tree.root],
			eachParent;

		//var matchesLength = matches.length;
		//for (var j = 0; j < matchesLength; j++) {
		//	editor.setNeedsRender(matches[j]);
		//}

		while ((eachParent = stack.pop())) {
			var eachChild = eachParent.firstChild;
			while (eachChild) {
				if (matchesAncestors.containsNode(eachChild)) {
					if (editor.isCollapsed(eachChild)) {
						// remove..
					}
					stack.push(eachChild);
				} else if (eachChild.firstChild) {
					if (!editor.isCollapsed(eachChild)) {
						editor.collapseNode(eachChild)[0].setIsManual(false);
					}
				}
				eachChild = eachChild.nextSibling;
			}
		}
	}

	function setNodePath(nodePath, restoringState) {
		// noRefresh only used at launch since folds are getting restored by
		// state and since don't want to run node path on unclassified tree

		var editor = this,
			oldNodeMatches = this._nodePathMatches;

		var isString = typeof nodePath === 'string' || nodePath instanceof String;
		if (isString) {
			this._nodePath = new NodePath(nodePath, editor.tree().taxonomy.types());
		} else {
			this._nodePath = nodePath;
		}

		this._nodePathMatches = null;
		this._nodePathMatchesNodeSet = null;
		this._nodePathEditedNodeSet = null;
		this._nodePathTopBoundaryFold = null;
		this._nodePathBottomBoundaryFold = null;

		if (restoringState) {
			editor.folds().forEach(function(each) {
				if (each.isBoundary()) {
					if (each.range().location() === 0) {
						editor._nodePathTopBoundaryFold = each;
					} else {
						editor._nodePathBottomBoundaryFold = each;
					}
				}
			});
		} else {
			editor.operation(function () {
				var viewport = editor.viewport();

				editor.removeFolds(editor.folds().filter(function (each) {
					return !each.isManual();
				}));

				editor.refresh();
				
				if (!editor.nodePath().isShowAllPath()) {
					if (true) {
						addFoldsToShowMatchesAndAncestors(editor);
					} else {
						addFoldsToShowMatchesAndAncestorsAndPreserveTreeStructure(editor);
					}
				}

				editor.scrollToLine(viewport.firstVisibleLine, viewport.firstVisibleLineOffset);
			});
		}

		if (editor._nodePathTopBoundaryFold || editor._nodePathBottomBoundaryFold) {
			// Revalidate selected range agains those top and bottom
			// boundaries
			editor.setSelectedRange(editor.selectedRange());
		}

		var outerThis = this;
		Extensions.processExtensions('com.foldingtext.editor.nodePathDidChange', function (each) {
			each(outerThis);
		});

		editor.invalidateRestorableState();
	}

	function nodePathLocation() {
		return this.nodePath().locationPathString();
	}

	function setNodePathLocation(location) {
		this.nodePath().setLocationPathString(location);
		this.setNodePath(this.nodePath());
	}

	function nodePathFilter() {
		return this.nodePath().filterStepString();
	}

	function setNodePathFilter(filter) {
		this.nodePath().setFilterStepString(filter);
		this.setNodePath(this.nodePath());
	}

	function treeChanged(editor, e) {
		if (!editor.nodePath().isShowAllPath()) {
			var deltas = e.deltas,
				nodePathEditedNodeSet = editor.nodePathEditedNodeSet(),
				each,
				i;

			for (i = 0; i < deltas.length; i++) {
				each = deltas[i];
				if (each.textChanged()) {
					nodePathEditedNodeSet.addNodeIgnoringNull(each.updatedNode);
					nodePathEditedNodeSet.addNodesFromArray(each.insertedNodes);
				}
			}
		}
	}

	Extensions.addState({
		readState: function (editor, state) {
			if (state.nodePath && state.nodePath !== NodePath.SHOW_ALL_PATH) {
				editor.tree().ensureClassified();
				editor.setNodePath(new NodePath(state.nodePath, editor.tree().taxonomy.types()), true);
			}
		},
		writeState: function (editor, state) {
			state.nodePath = editor.nodePath().nodePathString;
		}
		// process last so folds have already been loaded so can restore top
		// and bottom boundary folds
	}, Extensions.PriorityLast); 

	Extensions.add('com.foldingtext.editor.methods', /**@lends module:ft/editor/editor.Editor.prototype*/ {
		/**
		 * The object returned by this method is private and undocumented for
		 * the time begin, except you can call `toString()` on it to get the
		 * node path as a string.
		 * 
		 * @method
		 * @returns {Object} nodePath
		 * @methodgroup Node Path
		 */
		nodePath: nodePath,
		nodePathMatches: nodePathMatches,
		nodePathMatchesNodeSet: nodePathMatchesNodeSet,
		nodePathEditedNodeSet: nodePathEditedNodeSet,

		/**
		 * @method
		 * @param {String} nodePath - Node path string
		 */
		setNodePath: setNodePath,

		nodePathLocation: nodePathLocation,
		setNodePathLocation: setNodePathLocation,
		nodePathFilter: nodePathFilter,
		setNodePathFilter: setNodePathFilter
	});

	Extensions.addRenderNodeElementStyles(function (editor, node, elementRenderer) {
		if (!editor.nodePath().isShowAllPath()) {
			if (editor.nodePathMatchesNodeSet().containsNode(node)) {
				elementRenderer.addBackgroundClass('cm-match');
			} else if (editor.nodePathEditedNodeSet().containsNode(node)) {
				elementRenderer.addBackgroundClass('cm-notMatchEdited');
			} else {
				elementRenderer.addBackgroundClass('cm-notMatch');
			}
		}
	});

	//
	// Idea is to make top and bottom fold regions uneditable. Below code
	// makes those regions unselectable. But making them uneditable requires
	// more work... make each editing command aware of those regions. Only add
	// this back in once that work is done
	//

	/*Extensions.add('com.foldingtext.editor.beforeSelectionChange', function (editor, change) {		
		var topFold = editor._nodePathTopBoundaryFold,
			bottomFold = editor._nodePathBottomBoundaryFold;

		if (topFold && !topFold.valid()) {
			topFold = null;
		}

		if (bottomFold && !bottomFold.valid()) {
			bottomFold = null;
		}

		if (!topFold && !bottomFold) {
			return;
		}

		var cm = editor.cm(),
			lastLine = cm.lineCount() - 1,
			lastLineLastCh = cm.getLine(lastLine).length,
			topFind = topFold ? topFold._textMarker.find() : null,
			bottomFind = bottomFold ? bottomFold._textMarker.find() : null,
			minLineCh = topFind ? topFind.to : null,
			maxLineCh = bottomFind ? bottomFind.from : null;

		if (minLineCh && minLineCh.line < lastLine) {
			minLineCh.line++;
			minLineCh.ch = 0;
		}

		var minLine = minLineCh ? minLineCh.line : 0;
		if (maxLineCh && maxLineCh.line > minLine) {
			maxLineCh.line--;
			maxLineCh.ch = cm.getLine(maxLineCh.line).length;
		}

		var ranges = change.ranges,
			length = ranges.length,
			eachAnchor,
			eachHead;

		function constrain(lineCh) {
			var constrained = false;

			if (minLineCh) {
				if (lineCh.line < minLineCh.line) {
					lineCh.line = minLineCh.line;
					lineCh.ch = minLineCh.ch;
					constrained = true;
				} else if (lineCh.line === minLineCh.line && lineCh.ch < minLineCh.ch) {
					lineCh.ch = minLineCh.ch;
					constrained = true;
				}				
			}

			if (maxLineCh) {
				if (lineCh.line > maxLineCh.line) {
					lineCh.line = maxLineCh.line;
					lineCh.ch = maxLineCh.ch;
					constrained = true;
				} else if (lineCh.line === maxLineCh.line && lineCh.ch > maxLineCh.ch) {
					lineCh.ch = maxLineCh.ch;
					constrained = true;
				}				
			}

			return constrained;
		}

		var needsUpdate = false;

		for (var i = 0; i < length; i++) {
			needsUpdate |= constrain(ranges[i].anchor);
			needsUpdate |= constrain(ranges[i].head);
		}

		if (needsUpdate) {
			change.update(ranges);
		}
	});*/

	Extensions.addTreeChanged(treeChanged);

	Extensions.addCommand({
		name: 'home',
		performCommand: function (editor, options) {
			editor.operation(function () {
				var viewport = editor.viewport();
				editor.removeFolds(editor.folds());
				editor.setNodePath(NodePath.SHOW_ALL_PATH);
				editor.scrollToLine(viewport.firstVisibleLine, viewport.firstVisibleLineOffset);
			});
		}
	});

	Extensions.addCommand({
		name: 'link node path',
		description: 'Encode the selected node path text as clickable link.',
		performCommand: function (editor, options) {
			var nodepath = editor.selectedText(),
				encodedPath = encodeURIComponent(nodepath),
				applicationName = require('ft/util/environment').applicationName.toLowerCase();

			editor.replaceSelection('[label](x-' + applicationName + '://documents/current?nodepath=' + encodedPath + ')', 'start');
			editor.setSelectedRange(editor.selectedRange().rangeByOffsetting(1, 5));			
		}
	});
});