/*global DOMParser:true, XMLSerializer:true */
define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions,
		Node = require('ft/core/node').Node,
		dom = require('ft/util/dom');

	var OPMLTransform = function (tree) {
		this.tree = tree;
	};

	(function () {
		// add prototype functions here
		this.toOPMLObject = function toOPMLObject(rootNode) {
			var xmlDoc = document.implementation.createDocument(null, 'opml', null);

			var head = xmlDoc.createElement('head');
			var body = xmlDoc.createElement('body');
			var top = xmlDoc.firstChild;

			top.setAttribute('version', '2.0');
			top.appendChild(head);

			if (rootNode) {
				body.appendChild(this.OPMLOutlineFromNode(xmlDoc, rootNode, true));
			} else {
				var current = this.tree.root.firstChild;
				while (current) {
					var outlineEl = this.OPMLOutlineFromNode(xmlDoc, current, true);
					body.appendChild(outlineEl);
					current = current.nextSibling;
				}
			}

			top.appendChild(body);

			return xmlDoc;
		};

		this.OPMLOutlineFromNode = function OPMLOutlineFromNode(xmlDoc, node, isRecursive) {
			var outlineElement = document.createElementNS(null, 'outline');
			outlineElement.setAttribute('text', node.text());
			outlineElement.setAttribute('type', node.type());
			outlineElement.setAttribute('level', node.typeIndentLevel());
			var tags = node.tags();

			for (var tag in tags) {
				outlineElement.addAttribute(tag, tags[tag]);
			}

			if (isRecursive && node.hasChildren()) {
				var current = node.firstChild;
				while (current) {
					var childOutline = this.OPMLOutlineFromNode(xmlDoc, current, isRecursive);
					outlineElement.appendChild(childOutline);
					current = current.nextSibling;
				}
			}

			return outlineElement;
		};

		this.loadDocumentFromOPMLString = function loadDocumentFromOPMLString(string, insertBefore, replaceAll) {
			var opmlDoc = (new DOMParser()).parseFromString(string, 'text/xml');

			if (opmlDoc) {
				// just take all the outline nodes one by one and insert them in order
				var opmlDocRoot = opmlDoc.getElementsByTagName('opml');

				if (opmlDocRoot.length === 0) {
					return;
				} else {
					opmlDocRoot = opmlDocRoot[0];
				}

				var body = opmlDocRoot.getElementsByTagName('body');

				if (body.length === 0) {
					return;
				} else {
					body = body[0];
				}

				var curOutline = body.firstElementChild,
					curInsertionPoint = insertBefore;

				this.tree.beginUpdates();

				if (replaceAll) {
					this.tree.setText('');
					this.tree.root.removeChild(this.tree.root.firstChild);
					curInsertionPoint = this.tree.root;
				}

				var previous = null,
					nodeToBeInserted = null,
					beforeNode = null;
				while (curOutline) {
					nodeToBeInserted = this.tree.createNode(curOutline.getAttribute('text'), true);
					curInsertionPoint.appendChild(nodeToBeInserted);
					if (curOutline.getAttribute('level')) {
						nodeToBeInserted.setTypeIndentLevel(parseInt(curOutline.getAttribute('level'), 10));
					}

					if (curOutline.firstElementChild) {
						curInsertionPoint = nodeToBeInserted;
						curOutline = curOutline.firstElementChild;
					} else if (curOutline.nextElementSibling) {
						curOutline = curOutline.nextElementSibling;
					} else {
						var p = curOutline.parentElement,
							q = curInsertionPoint.parent;
						while (p) {
							if (!p.nextElementSibling) {
								p = p.parentElement;
								if (!q.parent) {
									curOutline = null;
									break;
								}
								q = q.parent;
							} else {
								curOutline = p.nextElementSibling;
								curInsertionPoint = q;
								break;
							}
						}
					}
				}

				this.tree.endUpdates();
			}
		};

	}).call(OPMLTransform.prototype);

	Extensions.addCommand({
		name: 'downloadOpml',
		performCommand: function (editor) {
			var ot = new OPMLTransform(editor.tree());
			var xmlDoc = ot.toOPMLObject();
			var string = (new XMLSerializer()).serializeToString(xmlDoc);
			var encodedString;
			encodedString = window.unescape(encodeURIComponent(string));
			var anchor = document.createElement('a');
			var date = new Date();
			var filename = 'oakoutliner' + [date.getMinutes(), date.getHours(), date.getDate(), date.getMonth()].join('-') + '.opml';
			anchor.setAttribute('download', filename);
			anchor.setAttribute('href', 'data:application/octet-stream;base64,' + window.btoa(encodedString));
			document.body.appendChild(anchor);
			var cancelled = true;
			if (document.createEvent) {
				var event = document.createEvent('MouseEvents');
				event.initMouseEvent('click', true, true, window,
					0, 0, 0, 0, 0,
					false, false, false, false,
					0, null);
				cancelled = !anchor.dispatchEvent(event);
			} else if (anchor.fireEvent) {
				cancelled = !anchor.fireEvent('onclick');
			}
		}
	});


	exports.OPMLTransform = OPMLTransform;
});

