define(function (require, exports, module) {
    'use strict';

    var dom = require('ft/util/dom'),
		Extensions = require('ft/core/extensions').Extensions,
		paneText = require('components/text/text!../../prefPane.html');

	var PrefPane = function (mainEditor) {
		var paneElement = document.getElementById('ui-pref-pane');
		if (!paneElement) {
			paneElement = document.createElement('div');
			paneElement.id = 'ui-pref-pane';
			paneElement.className = 'pane right slide';
			paneElement.innerHTML = paneText;
			document.body.appendChild(paneElement);
			var closeButton = document.getElementById('ui-pref-pane-close');
			closeButton.addEventListener('click', this.hidePreferences.bind(this));

			var saveButton = document.getElementById('ui-pref-pane-save');
			saveButton.addEventListener('click', this.savePreferencesToEditor.bind(this));
		}

		var enablePaneButton = document.getElementById('ui-pref-pane-show-btn');
		if (!enablePaneButton) {
			enablePaneButton = document.createElement('a');
			enablePaneButton.innerHTML = 'Preferences&hellip;';
			enablePaneButton.setAttribute('href', '#');
			enablePaneButton.className = 'jinnButton';
			document.body.appendChild(enablePaneButton);
			enablePaneButton.addEventListener('click', this.showPreferences.bind(this));
		}

		this.paneElement = paneElement;
		this.enablePaneButton = enablePaneButton;
		this._paneShowing = false;
		this._buttonVisible = false;
		this._mouseTracker = null;
		this._editor = mainEditor;

		this.loadPreferencesFromEditor(mainEditor);
		this.trackMouseForButtonVisibility();
	};

	(function () {
		this.trackMouseForButtonVisibility = function trackMouseForButtonVisibility() {
			var self = this;
			document.addEventListener('mousemove', function (e) {
				if (self._paneShowing) { return; }
				self.resetMouseTrackingTimer();
			});
		};

		this.resetMouseTrackingTimer = function resetMouseTrackingTimer() {
			var self = this;
			this.showEnablePaneButton();
			if (this._mouseTracker) {
				clearTimeout(this._mouseTracker);
			}

			this._mouseTracker = setTimeout(function () {
				self.hideEnablePaneButton();
				self._mouseTracker = null;
			}, 1000);
		};

		this.showEnablePaneButton = function showEnablePaneButton() {
			dom.addClass(this.enablePaneButton, 'visible');
		};

		this.hideEnablePaneButton = function hideEnablePaneButton() {
			dom.removeClass(this.enablePaneButton, 'visible');
		};

		this.showPreferences = function showPreferences() {
			dom.addClass(this.paneElement, 'visible');
			this.loadPreferencesFromEditor(this._editor);
			this._paneShowing = true;
		};

		this.hidePreferences = function hidePreferences() {
			dom.removeClass(this.paneElement, 'visible');
			this._paneShowing = false;
		};

		this.loadPreferencesFromEditor = function loadPreferencesFromEditor(editor) {
			// load preferences
			var columnWidthEl = document.getElementById('pref-columnwidth');
			var columnWidthOption = editor.option('wrapToColumn');
			columnWidthEl.value = parseInt(columnWidthOption, 10);

			// load typewriter mode
			var typewriterModeEl = document.getElementById('pref-typewriter');
			var typewriterOption = editor.option('typewriterMode');
			typewriterOption.checked = typewriterOption;
		};

		this.savePreferencesToEditor = function savePreferencesToEditor() {
			// load preferences
			var columnWidthEl = document.getElementById('pref-columnwidth');
			this._editor.setOption('wrapToColumn', parseInt(columnWidthEl.value, 10));


			// load typewriter mode
			var typewriterModeEl = document.getElementById('pref-typewriter');
			this._editor.setOption('typewriterMode', typewriterModeEl.checked);
		};

	}).call(PrefPane.prototype);


	/*Extensions.addInit(function (editor) {
		editor._prefPane = new PrefPane(editor);
	});*/

});