define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions,
		CodeMirror = require('codemirror').CodeMirror,
		Node = require('ft/core/node').Node,
		replaceLocationRE = /[\-+*]/,
		shouldRenderNode = function shouldRenderNode(node) {
			return node.type() === 'unordered';
		};

	Extensions.add('com.foldingtext.editor.options', {
		name: 'showListBullets',
		defaultValue: true,
		onChange: function (editor, oldValue, newValue) {
			if (oldValue === newValue) {
				return;
			}

			editor.operation(function () {
				var each = editor.tree().firstLineNode();
				while (each) {
					if (shouldRenderNode(each)) {
						editor.setNeedsRender(each);
					}
					each = each.nextLineNode();
				}
				editor.refresh();
			});
		}
	});

	Extensions.addCommand({
		name: 'toggleShowListBullets',
		validate: function (editor, options) {
			options.interfaceItemState = editor.option('showListBullets');
			return true;
		},
		performCommand: function (editor) {
			editor.setOption('showListBullets', !editor.option('showListBullets'));
		}
	});

	Extensions.addRenderNode(function (editor, node, nodeRenderer) {
		if (editor.option('showListBullets') && shouldRenderNode(node)) {
			var match = replaceLocationRE.exec(node.line());
			if (match) {
				var widget =  document.createElement('span');
				widget.className = 'cm-list-bullet';
				widget.textContent = '•';
				nodeRenderer.renderInlineWidget(match.index, 1, widget);
			}
		}
	});

	function configure(shouldRenderNodeOverride, createArchiveNodeOverride) {
		if (shouldRenderNodeOverride) {
			shouldRenderNode = shouldRenderNodeOverride;			
		}
		if (createArchiveNodeOverride) {
			createArchiveNode = createArchiveNodeOverride;
		}
	}

	return configure;
});