define(function (require, exports, module) {
	'use strict';

	require('ft/extensions/indent');
	require('ft/extensions/folding');
	require('ft/extensions/foldingnodes');

	// require to be sure that basic 'insertNewLine' gets installed first
	// then we install ourselves over that.
	require('ft/editor/commands');

	var Extensions = require('ft/core/extensions').Extensions;

	function smartInsertNewline(editor) {
		var tree = editor.tree(),
			selectedRange = editor.selectedRange(),
			node = selectedRange.startNode,
			currentLeadMatch = node.tree.taxonomy.matchLeadForNode(node),
			currentLead = currentLeadMatch[0],
			nextLead = '',
			after = true;

		if (selectedRange.isCollapsed()) {
			if (currentLead.length !== 0 &&
				selectedRange.startOffset === currentLead.length &&
				currentLead === node.line()) {

				editor.operation(function () {
					editor.setSelectedRange(tree.createRangeFromNodes(node, 0, node, node.line().length));
					editor.replaceSelection('', 'end', '+input');
					editor.scrollRangeToVisible(editor.selectedRange());
				});

				return;
			}

			var lineNumber = editor.tree().nodeToLineNumber(node),
				matchNode;

			if (editor.lineIsHiddenInFold(lineNumber)) {
				while (lineNumber > 0 && editor.lineIsHiddenInFold(lineNumber)) {
					lineNumber--;
				}
				matchNode = tree.lineNumberToNode(lineNumber);
			} else {
				if (selectedRange.startOffset <= currentLeadMatch[0].length) {
					selectedRange.startOffset = 0;
					selectedRange.endOffset = 0;
					editor.operation(function () {
						editor.replaceTextInRange(selectedRange, '\n', '+input');
						editor.scrollRangeToVisible(editor.selectedRange());
					});
					return;
				}

				editor.operation(function () {
					editor.expandToRevealNode(node);
					editor.expandNode(node);
					editor.setSelectedRange(selectedRange);
				});

				if (node.firstChild) {
					matchNode = node.firstChild;
					after = false;
				} else {
					matchNode = node;
				}
			}
			nextLead = matchNode.tree.taxonomy.nextLeadFromNode(matchNode, after);
		}

		editor.operation(function () {
			editor.replaceSelection('\n' + nextLead, 'end', '+input');
			editor.scrollRangeToVisible(editor.selectedRange());
		});
	}

	Extensions.addCommand({
		name: 'smartInsertNewline',
		performCommand: smartInsertNewline
	});

	Extensions.addCommand({
		name: 'smartInsertNewlineAbove',
		performCommand: function (editor) {
			editor.performCommand('moveToBeginningOfParagraph');
			editor.performCommand('smartInsertNewline');
			editor.performCommand('moveLeft');
		}
	});

	Extensions.addCommand({
		name: 'smartInsertNewlineAfter',
		performCommand: function (editor) {
			editor.performCommand('moveToEndOfParagraph');
			editor.performCommand('smartInsertNewline');
		}
	});
});