define(function (require, exports, module) {
	'use strict';

	require('ft/util/string');

	var Extensions = require('ft/core/extensions').Extensions,
		wordRegex = /\w/;

	function processSnippetDictionary(potentialSnippetText, snippetDictionary) {
		for (var each in snippetDictionary) {
			if (potentialSnippetText.endsWith(each)) {
				var beforeSnippetChar = potentialSnippetText[potentialSnippetText.length - (each.length + 1)],
					startSnippetChar = each[0];

				if (beforeSnippetChar && wordRegex.test(beforeSnippetChar) && wordRegex.test(startSnippetChar)) {
					return;
				}

				return {
					shortcut: each,
					expansion: snippetDictionary[each]
				};
			}
		}
	}

	Extensions.addCommand({
		name: 'replaceTextSnippet',
		performCommand: function (editor) {
			var selectedRange = editor.selectedRange();
			if (selectedRange.isCollapsed()) {
				var node = selectedRange.startNode,
					line = node.line(),
					snippetEnd = selectedRange.startOffset,
					snippetStart = snippetEnd;

				while (snippetStart > 0 && line[snippetStart - 1] !== ' ') {
					snippetStart--;
				}

				var tree = editor.tree(),
					shortcutRange = tree.createRangeFromNodes(node, snippetStart, node, snippetEnd),
					potentialSnippetText = shortcutRange.textInRange();

				if (potentialSnippetText) {
					var snippet = Extensions.processExtensions('com.foldingtext.editor.snippets', function (eachSnippetDictionary) {
						return processSnippetDictionary(potentialSnippetText, eachSnippetDictionary);
					}, true);

					if (!snippet) {
						if (window.nativeViewController) {
							/*jshint camelcase: false */
							snippet = processSnippetDictionary(potentialSnippetText, JSON.parse(window.nativeViewController.javascriptUserReplacementsDictionary()));
							/*jshint camelcase: true */
						}
					}

					if (snippet) {
						var shortcut = snippet.shortcut,
							expansion = snippet.expansion;

						if (typeof(expansion) === 'function') {
							expansion = expansion(shortcut);
						}

						editor.replaceTextInRange(selectedRange.rangeByOffsetting(-shortcut.length, shortcut.length), expansion);
						return true;
					}
				}
			}
			return false;
		}
	});
});