// To complicatd for normal use. Maybe add as a plugin later.
define(function (require, exports, module) {
	'use strict';

	var Extensions = require('ft/core/extensions').Extensions;

	function indentStructure(editor) {
		var cm = editor.cm(),
			pos = cm.getCursor(),
			line = cm.getLine(pos.line),
			lineOrigin = {line: pos.line, ch: 0},
			afterState = cm.getStateAfter(pos.line),
			lead, leadingSpace, leadingHandle, maxIndentPos, nextLeadingSpace, nextHandle,
			match, indentPerformed = false,
			nextHandleAfter = function (currentHandle) {
				switch (currentHandle) {
				case '-':
					return '*';
				case '*':
					return '+';
				case '+':
					return '-';
				}

				return currentHandle;
			};

		switch (afterState.type) {
		case 'heading':
			var headingCount = afterState.heading;	// FIX: state.heading auto clamps between 1,6 (however, user can enter more than 6)
			if (pos.ch < headingCount + 2) {
				cm.replaceRange('#', lineOrigin, lineOrigin);
				indentPerformed = true;
			}
			break;
		case 'unordered':
			lead = /(\s*)([\-+*])\s/.exec(line);
			leadingSpace = lead[1];
			leadingHandle = lead[2];
			maxIndentPos = leadingSpace.length + leadingHandle.length + 2;

			if (pos.ch < maxIndentPos) {
				cm.replaceRange('\t', lineOrigin, lineOrigin);
				indentPerformed = true;
			}

			break;
		case 'ordered':
			lead = /(\s*)([0-9]+\.)\s/.exec(line);
			leadingSpace = lead[1];
			leadingHandle = lead[2];
			maxIndentPos = leadingSpace.length + leadingHandle.length + 2;
			nextHandle = '1.';

			if (pos.ch < maxIndentPos) {
				cm.replaceRange('\t' + nextHandle,
					{line: pos.line, ch: leadingSpace.length},
					{line: pos.line, ch: leadingSpace.length + leadingHandle.length});
				indentPerformed = true;
			}
			break;
		case 'blockquote':
			lead = /^([> ]+)/.exec(line);
			leadingHandle = lead[1];
			maxIndentPos = leadingHandle.length + 1;

			if (pos.ch < maxIndentPos) {
				cm.replaceRange('> ', lineOrigin, lineOrigin);
				indentPerformed = true;
			}
			break;
		}

		if (!indentPerformed) {
			cm.replaceSelection('\t', 'end');
		}
	}

	function unindentStructure(editor) {
		var cm = editor.cm(),
			pos = cm.getCursor(),
			line = cm.getLine(pos.line),
			afterState = cm.getStateAfter(pos.line),
			lead, leadingSpace, leadingHandle, maxIndentPos, nextHandle, nextLeadingSpace,
			match, indentPerformed = false,
			lineOrigin = {line: pos.line, ch: 0},
			lineDeleteDestination = {line: pos.line, ch: 1},
			firstIndent = /^(\t|[ ]{4})/.exec(line),
			nextHandleAfter = function (currentHandle) {
				switch (currentHandle) {
				case '*':
					return '-';
				case '+':
					return '*';
				case '-':
					return '+';
				}

				return currentHandle;
			};

		if (!firstIndent) {
			lineDeleteDestination = lineOrigin;
		} else {
			var numChars = firstIndent[1].length;
			if (numChars > 1) {
				lineDeleteDestination.ch = numChars;
			}
		}

		switch (afterState.type) {
		case 'heading':
			var headingCount = afterState.heading;	// FIX: state.heading auto clamps between 1,6 (however, user can enter more than 6)
			if (pos.ch < headingCount + 2) {
				cm.replaceRange('', lineOrigin, {line: pos.line, ch: 1});
				indentPerformed = true;
			}
			break;
		case 'unordered':
			if (firstIndent) {
				lead = /(\s*)([\-+*])\s/.exec(line);
				leadingSpace = lead[1];
				leadingHandle = lead[2];
				maxIndentPos = leadingSpace.length + leadingHandle.length + 2;
				if (pos.ch < maxIndentPos) {
					cm.replaceRange('',
						lineOrigin,
						lineDeleteDestination);
					indentPerformed = true;
				}
			}
			break;
		case 'ordered':
			if (firstIndent) {
				lead = /(\s*)([0-9]+\.)\s/.exec(line);
				leadingSpace = lead[1];
				leadingHandle = lead[2];
				maxIndentPos = leadingSpace.length + leadingHandle.length + 2;

				if (pos.ch < maxIndentPos) {
					cm.replaceRange('',
						lineOrigin,
						lineDeleteDestination);
					indentPerformed = true;
				}
			}
			break;
		case 'blockquote':
			lead = /^([> ]+)/.exec(line);
			leadingHandle = lead[1];
			maxIndentPos = leadingHandle.length + 1;

			if (pos.ch < maxIndentPos) {
				cm.replaceRange('',
					lineOrigin,
					{line: pos.line, ch: 2});
				indentPerformed = true;
			}
			break;
		}

		if (!indentPerformed) {
			cm.execCommand('moveParagraphsLeft');
		}
	}

	Extensions.add('com.foldingtext.editor.options', {
		name: 'structureindent',
		defaultValue: false,
		onChange: function (editor, oldValue, newValue) {
			if (oldValue) { editor.removeKeyMap('structureindent'); }
			if (newValue) { editor.addKeyMap({
					name: 'structureindent',
					'Tab': 'indentStructure',
					'Shift-Tab': 'unindentStructure'
				});
			}
		}
	});

	Extensions.addCommand({
		name: 'indentStructure',
		performCommand: indentStructure
	});

	Extensions.addCommand({
		name: 'unindentStructure',
		performCommand: unindentStructure
	});
});