//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//

/** @module ft/system/notificationcenter */
define(function (require, exports, module) {
	'use strict';

	/**
	 * Deliver system level notifications to the user.
	 *
	 * @private
	 * @constructor
	 * @alias module:ft/system/notificationcenter.NotificationCenter*/
	var NotificationCenter = function NotificationCenter() {
	};

	/**
	 * Deliver a system level notification to the user.
	 *
	 * @param {String}
	 * @param {String}
	 * @param {String}
	 */
	NotificationCenter.deliverNotification = function deliverNotification(title, subtitle, informativeText) {
		if (window.nativeViewController) {
			/*jshint camelcase: false */
			window.nativeViewController.javascriptDeliverNotification_(JSON.stringify({
				title : title,
				subtitle : subtitle,
				informativeText : informativeText
			}));
			/*jshint camelcase: true */
		} else {
			var Notification = window.Notification;
			if (Notification) {
				if (Notification.permission === 'default') {
					Notification.requestPermission(function () {
					});
				} else if (Notification.permission === 'granted') {
					subtitle = subtitle ? subtitle + '\n' : '';
					informativeText = informativeText ? informativeText : '';

					var n = new Notification(title, {
						'body': subtitle + informativeText
					});
				}
			}
		}
	};

	exports.NotificationCenter = NotificationCenter;
});