//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//

/** 
 * @module ft/system/userdefaults
 */
define(function (require, exports, module) {
	'use strict';

	/**
	 * UserDefaults allows you to read and write application level user
	 * default values. See {@link module:ft/core/extensions.Extensions.addState|addState}
	 * if you want to store state along with a document.
	 *
	 * @private
	 * @constructor
	 * @alias module:ft/system/userdefaults.UserDefaults*/
	var UserDefaults = function UserDefaults() {
	};

	/**
	 * Read from the user defaults.
	 *
	 * @param {String} - User defaults key.
	 * @returns {String}
	 */
	UserDefaults.stringForKey = function stringForKey(key) {
		if (window.nativeViewController) {
			/*jshint camelcase: false */
			var result = window.nativeViewController.javascriptUserDefaultsStringForKey_(key);
			/*jshint camelcase: true */
			return result;
		}
		return undefined;
	};

	/**
	 * Write to the user defaults.
	 *
	 * @param {String} - New defaults value.
	 * @param {String} - Key to read value.
	 */
	UserDefaults.setStringForKey = function setStringForKey(string, key) {
		if (window.nativeViewController) {
			/*jshint camelcase: false */
			window.nativeViewController.javascriptUserDefaultsSetString_forKey_(string, key);
			/*jshint camelcase: true */
		}
	};

	exports.UserDefaults = UserDefaults;
});