//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	/*jshint regexp: false */
	var amountsRE = /\b(\d+(?:.?\d+)?)\s?((?:second|sec|minute|min|hour|day|week|year)s?)\b/g;
	/*jshint regexp: true */

	function parseDuration(text, state, previousState) {
		var escapedText = text.escapedText(),
			duration = 0,
			match,
			i = 0;

		while ((match = amountsRE.exec(escapedText))) {
			var amountString = match[0],
				quantityString = match[1],
				unitString = match[2],
				quantity = parseFloat(quantityString);

			text.addAttributeInRange(
				'amount',
				amountString,
				i + match.index,
				amountString.length);
			text.addAttributeInRange(
				'quantity',
				quantityString,
				i + match.index,
				quantityString.length);
			text.addAttributeInRange(
				'unit',
				unitString,
				i + match.index + amountString.length - unitString.length,
				unitString.length);

			switch (unitString) {
			case 'second':
			case 'seconds':
			case 'sec':
			case 'secs':
				duration += quantity;
				break;
			case 'minute':
			case 'minutes':
			case 'min':
			case 'mins':
				duration += (quantity * 60);
				break;
			case 'hour':
			case 'hours':
				duration += (quantity * 60 * 60);
				break;
			case 'day':
			case 'days':
				duration += (quantity * 60 * 60 * 24);
				break;
			case 'week':
			case 'weeks':
				duration += (quantity * 60 * 60 * 24 * 7);
				break;
			case 'year':
			case 'years':
				duration += (quantity * 60 * 60 * 24 * 360);
				break;
			}
		}
		return duration;
	}

	return {
		parseDuration: parseDuration,
		attributesToClear: ['amount', 'quantity', 'unit']
	};
});