//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var CodeMirror,
		codeMirrorModulePath = 'codemirror';

	if (require.defined(codeMirrorModulePath)) {
		CodeMirror = require(codeMirrorModulePath).CodeMirror;
	}

//	CodeMirror.defineMIME('json', {name: "javascript", json: true});

	var modeCache = {},
		ObjectUtils = require('ft/util/objectutils');

	function getMode(options, mimeType) {
		var mode = modeCache[mimeType];
		if (mode === undefined) {
			mode = CodeMirror.getMode(options || {}, mimeType);
			if (mode.name === 'null') {
				mode = CodeMirror.getMode(options || {}, 'text/' + mimeType);
				if (mode.name === 'null') {
					mode = CodeMirror.getMode(options || {}, 'text/x-' + mimeType);
					if (mode.name === 'null') {
						mode = CodeMirror.getMode(options || {}, 'application/' + mimeType);
					}
				}
			}
			if (!mode) {
				mode = null;
			}
			modeCache[mimeType] = mode;
		}
		return mode;
	}

	function runMode(mode, state, text) {
		var stream = text.escapedTextStream();
		while (!stream.eol()) {
			stream.start = stream.pos;
			var style = mode.token(stream, state);
			if (style) {
				var styles = style.split(' '),
					value = stream.current(),
					length = styles.length,
					i;

				for (i = 0; i < length; i++) {
					text.addAttributeInRange('mode-' + styles[i], value, stream.start, stream.pos - stream.start);
				}
			}
		}
	}

	function copyState(state) {
		if (!state) {
			return undefined;
		}
		return {
			mode : state.mode,
			state : state.state ? CodeMirror.copyState(state.mode, state.state) : null
		};
	}

	function stateEquals(a, b) {
		if (a === b) {
			return true;
		}
		if (!a || !b) {
			return false;
		}
		if (a.mode !== b.mode) {
			return false;
		}
		return ObjectUtils.shallowEquals(a.state, b.state);
	}

	if (CodeMirror) {
		return {
			getMode: getMode,
			runMode: runMode,
			copyState: copyState,
			stateEquals: stateEquals,
			attributesToClear: [
				'mode-keyword',
				'mode-atom',
				'mode-number',
				'mode-def',
				'mode-variable',
				'mode-variable-2',
				'mode-variable-3',
				'mode-property',
				'mode-operator',
				'mode-comment',
				'mode-string',
				'mode-string-2',
				'mode-meta',
				'mode-error',
				'mode-qualifier',
				'mode-builtin',
				'mode-bracket',
				'mode-tag',
				'mode-attribute',
				'mode-heading',
				'mode-quote',
				'mode-hr',
				'mode-link',
				'mode-negative',
				'mode-positive',
				'mode-heading',
				'mode-strong',
				'mode-em',
				'mode-link',
				'mode-invalidchar'
			]
		};
	} else {
		return {
			getMode: function () {
				return null;
			},
			runMode: function () {
			},
			copyState: function () {
				return undefined;
			},
			stateEquals: function () {
				return true;
			},
			attributesToClear: []
		};
	}
});