//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	require('ft/extensions/formatting');

	var Extensions = require('ft/core/extensions').Extensions;

	function matchAndHighlightInTextFromIndex(text, t, i, found, looking, style) {
		var end = t.indexOf(looking, i + found.length);
		if (end !== -1) {
			text.addAttributeInRange(style, text.textSubstring(i + found.length, end), i, (end + looking.length) - i);
			text.addAttributeInRange('criticstart', '{', i, 1);
			text.addAttributeInRange('keyword', found, i, found.length);
			text.addAttributeInRange('keyword', looking, end, looking.length);
			text.addAttributeInRange('criticend', '}', (end + looking.length) - 1, 1);
			return end + looking.length;
		}
		return i + found.length;
	}

	function parseCriticMarkup(text, state, previousState) {
		var t = text.escapedText(),
			i = t.indexOf('{'),
			ch1, ch2,
			end;

		while (i !== -1) {
			ch1 = t[i + 1];
			ch2 = t[i + 2];
			if (ch1 === '+' && ch2 === '+') {
				i = matchAndHighlightInTextFromIndex(text, t, i, '{++', '++}', 'criticinsertion');
			} else if (ch1 === '-' && ch2 === '-') {
				i = matchAndHighlightInTextFromIndex(text, t, i, '{--', '--}', 'criticdeletion');
			} else if (ch1 === '~' && ch2 === '~') {
				end = t.indexOf('~~}', i + 3);
				if (end !== -1) {
					var middle = t.indexOf('~>', i);
					if (middle !== -1 && middle < end) {
						matchAndHighlightInTextFromIndex(text, t, i, '{~~', '~~}', 'criticsubstitution');
						text.addAttributeInRange('criticdeletion', text.textSubstring(i + 3, middle), i + 3, (middle - (i + 3)));
						text.addAttributeInRange('keyword', '~>', middle, 2);
						text.addAttributeInRange('criticinsertion', text.textSubstring(i + 3, middle), middle + 2, (end - middle) - 2);
						i = end + 3;
					} else {
						i = i + 3;
					}
				} else {
					i = i + 3;
				}
			} else if (ch1 === '>' && ch2 === '>') {
				i = matchAndHighlightInTextFromIndex(text, t, i, '{>>', '<<}', 'criticcomment');
			} else if (ch1 === '=' && ch2 === '=') {
				i = matchAndHighlightInTextFromIndex(text, t, i, '{==', '==}', 'critichighlight');
			} else {
				i = t.indexOf('{', i + 1);
			}
		}
	}

	Extensions.addCommand({
		name: 'toggleAddition',
		performCommand: function (editor) {
            editor.toggleSelectionFormatting(editor, '{++', '++}');
		}
	});

	Extensions.addCommand({
		name: 'toggleDeletion',
		performCommand: function (editor) {
            editor.toggleSelectionFormatting(editor, '{--', '--}');
		}
	});

	Extensions.addCommand({
		name: 'toggleSubstitution',
		performCommand: function (editor) {
            if (editor.toggleSelectionFormatting(editor, '{~~', '~~}')) {
				var range = editor.selectedRange().rangeByCollapsing(false);
				editor.setSelectedRange(range);
				editor.replaceSelection('~>');
            } else {

            }
		}
	});

	Extensions.addCommand({
		name: 'toggleComment',
		performCommand: function (editor) {
            editor.toggleSelectionFormatting(editor, '{>>', '<<}');
		}
	});

	Extensions.addCommand({
		name: 'toggleHighlight',
		performCommand: function (editor) {
            editor.toggleSelectionFormatting(editor, '{==', '==}');
		}
	});

	Extensions.add('com.foldingtext.editor.clearInlineFormatting', function (formattedText) {
		var format = formattedText.format();
		if (format === 'criticinsertion' ||
			format === 'criticdeletion' ||
			format === 'criticsubstitution' ||
			format === 'criticcomment' ||
			format === 'critichighlight') {

			formattedText.replaceInRange('', formattedText.length() - 3, 3);
			formattedText.replaceInRange('', 0, 3);

			if (format === 'criticsubstitution') {
				var index = formattedText.string().indexOf('~>');
				if (index !== -1) {
					formattedText.replaceInRange('', index, 2);
				}
			}
		}
	});
	
	return {
		parseCriticMarkup: parseCriticMarkup,
		attributesToClear: ['criticinsertion', 'criticdeletion', 'criticsubstitution', 'criticcomment', 'critichighlight']
	};
});
