//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

//	in process of moving htmlState to share space with codemirrorState

	var CodeMirrorModeHelper = require('ft/taxonomy/helpers/codemirrormodehelper'),
		ObjectUtils = require('ft/util/objectutils'),
		/*jshint regexp: false */
		htmlTagRegex = /<(\/)?\w+.*(\/)?>/g,
		htmlOpenTagRegex = /<\w+.*(\/)?>/,
		htmlCommentTagRegex = /<!--.*-->/,
		/*jshint regexp: true */
		InlinesHelper;

	function matchXMLBlockStart(text) {
		if (text.indexOf('>') !== -1) {
			var match = htmlOpenTagRegex.exec(text);
			if (match) {
				InlinesHelper.emailAutoLinkReplaceRegex.lastIndex = 0;
				InlinesHelper.webAutoLinkReplaceRegex.lastIndex = 0;
				if (InlinesHelper.webAutoLinkReplaceRegex.test(match[0])) {
					return null;
				}
				if (InlinesHelper.emailAutoLinkReplaceRegex.test(match[0])) {
					return null;
				}
				return match;
			} else {
				return htmlCommentTagRegex.exec(text);
			}
		}
		return null;
	}

	function matchAnyXMLTag(text) {
		var match,
			i = 0;
		while ((match = htmlTagRegex.exec(text))) {
			var matchText = match[0];
			InlinesHelper.emailAutoLinkReplaceRegex.lastIndex = 0;
			InlinesHelper.webAutoLinkReplaceRegex.lastIndex = 0;
			if ((InlinesHelper.emailAutoLinkReplaceRegex.test(matchText))) {
				continue;
			} else if (InlinesHelper.webAutoLinkReplaceRegex.test(matchText)) {
				continue;
			} else {
				htmlTagRegex.lastIndex = 0; // reset regex for next time
				return match;
			}
		}
		return null;
	}

	function shouldRunHTMLMode(text, codemirrorState, forceParseFromStart) {
		var escapedText = text.escapedText(),
			htmlPossible;

		if (forceParseFromStart) {
			if (escapedText[0] === '<') {
				var match = matchXMLBlockStart(escapedText);
				return !!match && match.index === 0;
			}
		} else {
			if (!!codemirrorState) {
				return true;
			}
			if (escapedText.indexOf('<') !== -1) {
				return !!matchAnyXMLTag(escapedText);
			}
		}
		return false;
	}

	function parseHTML(text, codemirrorState, forceParseFromStart, htmlMode) {
		if (!htmlMode || !shouldRunHTMLMode(text, codemirrorState, forceParseFromStart)) {
			return null;
		}

		if (!codemirrorState) {
			codemirrorState = {
				mode: htmlMode,
				state: htmlMode.startState()
			};
		}

		CodeMirrorModeHelper.runMode(htmlMode, codemirrorState.state, text);

		return codemirrorState;
	}

	function detectAndHighlightHtmlBlock(text, state, previousState, htmlMode) {
		var forceParseFromStart = false,
			codemirrorState;

		if (previousState.codemirrorState) {
			codemirrorState = CodeMirrorModeHelper.copyState(previousState.codemirrorState);
		} else {
			if (previousState.type !== 'empty' && !text.isFirstLine()) {
				return false;
			} else {
				forceParseFromStart = true;
			}
		}

		if (!InlinesHelper) {
			InlinesHelper = require('ft/taxonomy/helpers/inlineshelper');
		}

		codemirrorState = parseHTML(text, codemirrorState, forceParseFromStart, htmlMode);

		if (codemirrorState) {
			state.type = 'htmlblock';
			state.codemirrorState = codemirrorState;
			return true;
		} else {
			return false;
		}
	}

	return {
		detectAndHighlightHtmlBlock: detectAndHighlightHtmlBlock,
		parseHTML: parseHTML,
		attributesToClear: []
	};
});