//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	function allowsMagicLinkAtLocation(text, location) {
		return true;

		/* This is a performance killer, breaking the lazy attribute add optionization
		in attributedString var attributes = text.attributesAtIndex(location);
		return  !attributes.inlinelink && !attributes.autolink &&
		!attributes.linktarget;*/
	}


	function parseScheme(text) {
		if (text.indexOf('www') === 0) {
			return 'http';
		}
		var colonIndex = text.indexOf(':');
		if (colonIndex !== -1) {
			return text.substring(0, colonIndex);
		}
		return undefined;
	}

	var emailRegexp = /\b[A-Z0-9\._%+\-]+@[A-Z0-9\.\-]+\.[A-Z]{2,4}\b/gi;

	function parseEmailLinks(text, state, previousState) {
		var escapedText = text.escapedText(),
			match;

		if (escapedText.indexOf('@') === -1) {
			return;
		}

		while ((match = emailRegexp.exec(escapedText))) {
			var linkStart = match.index,
				emailLink = match[0];

			if (escapedText[linkStart - 1] === ':') {
				// hack... if there's a scheme then return, will be caught by parseWebLinks
				return;
			}

			if (allowsMagicLinkAtLocation(text, linkStart)) {
				text.addAttributeInRange('magiclink', emailLink, linkStart, emailLink.length);
				text.addAttributeInRange('linktarget', 'mailto:' + emailLink, linkStart, emailLink.length);
			}
		}
	}

	// http://stackoverflow.com/questions/2026041/help-hacking-grubers-liberal-url-regex

	var webRegexp = /\b(?:([a-z][\w\-]+:(?:\/{1,3}|[a-z0-9%])|www\d{0,3}[.])(?:[^\s()<>]+|\([^\s()<>]+\))+(?:\([^\s()<>]+\)|[^`!()\[\]{};:'".,<>?«»“”‘’\s]))/gi;

	function parseWebLinks(text, state, previousState) {
		var escapedText = text.escapedText(),
			match;

		webRegexp.lastIndex = 0;
		while ((match = webRegexp.exec(escapedText))) {
			var linkStart = match.index,
				linkText = match[0],
				linkTarget = linkText,
				scheme = parseScheme(linkText);

			if (allowsMagicLinkAtLocation(text, linkStart)) {
				if (escapedText.indexOf('www') === 0) {
					linkTarget = 'http://' + linkText;					
				}
				text.addAttributeInRange('magiclink', linkText, linkStart, linkText.length);
				text.addAttributeInRange('linktarget', linkTarget, linkStart, linkText.length);
			}
		}
	}

	return {
		parseScheme: parseScheme,
		parseEmailLinks: parseEmailLinks,
		parseWebLinks: parseWebLinks,
		attributesToClear: ['magiclink', 'linktarget']
	};
});