//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	/*jshint regexp: false */
	var propertyTypeRegex = /^\s*([^\s]*)\s:(?:\s)(.*)/;
	/*jshint regexp: true */

	function parsePropertyType(text, state, previousState) {
		var match;
		if (text.text().indexOf(' : ') !== -1 && (match = text.textMatch(propertyTypeRegex))) {
			var propertyName = match[1],
				propertyValue = match[2];

			state.type = 'property';
			state.typeIndentLevel = state.logicalIndent;
			state.tags.name = propertyName.toLowerCase();
			state.tags.value = propertyValue;
			text.addAttributeInRange('propertyname', propertyName, 0, propertyName.length);
			text.addAttributeInRange('keyword', ':', propertyName.length + 1, 1);

			if (propertyValue) {
				text.addAttributeInRange('propertyvalue', propertyValue, propertyName.length + 3, propertyValue.length);
			}
			return true;
		}
		return false;
	}

	function setPropertyNodeValue(node, propertyValue) {
		try {
			var text = node.lineAttributedString().string();
			var valueStart = text.indexOf(': ') + 2;
			var valueLength = text.length - valueStart;
			node.replaceLineInRange(propertyValue, valueStart, valueLength);
		} catch (e) {
			return false;
		}
	}

	return {
		setPropertyNodeValue: setPropertyNodeValue,
		parsePropertyType: parsePropertyType,
		attributesToClear: ['propertyname', 'propertyvalue']
	};
});
