//
// Copyright (c) 2014 Jesse Grosjean. All rights reserved.
//
// Permission is given for you to use this software ONLY while creating themes
// and plugins for the "FoldingText" software platform. Permission is NOT
// given for any other purpose.
//
define(function (require, exports, module) {
	'use strict';

	var Classifier = require('ft/core/classifier').Classifier,
		Extensions = require('ft/core/extensions').Extensions,
		Taxonomy = require('ft/core/taxonomy').Taxonomy,
		oop = require('ft/util/oop');

	var TestTaxonomy = function (spec) {
		Taxonomy.call(this, spec);
	};

	oop.inherits(TestTaxonomy, Taxonomy);

	(function () {

		var classifyState = 0;

		this.classify = function classify(text, state, previousState) {
			if (classifyState === 0) {
				state.typeIndentLevel = 401;
			} else if (classifyState === 1) {
				state.typeIndentLevel = 402;
			} else if (classifyState === 2) {
				state.typeIndentLevel = 400;
			} else if (classifyState === 3) {
				state.typeIndentLevel = 401;
				state.nextAction = Classifier.RECLASSIFY_PREVIOUS;
			}

			classifyState++;

//			state.typeIndentLevel = Math.random();
/*			state.typeIndentLevel = level++;
			if (level > 10000) {
				level = 0;
			}*/
		};


		this.reclassifyUp = function reclassifyUp(text, state, nextState) {
			if (classifyState === 4) {
				state.typeLevel = nextState.typeLevel;
				state.typeIndentLevel = nextState.typeIndentLevel;
				state.nextAction = Classifier.RECLASSIFY_PREVIOUS;
				classifyState++;
			} else {
				state.nextAction = Classifier.CLASSIFY_NEXT;
				classifyState = 1;
			}
		};

		this.reclassifyDown = function reclassifyDown(text, state, previousState) {
		};
		
		this.startState = function startState() {
			return {
				typeLevel: 0,
				typeIndentLevel: 0,
				nextAction: Classifier.CLASSIFY_NEXT
			};
		};

		this.copyState = function copyState(state) {
			return {
				typeLevel: state.typeLevel,
				typeIndentLevel: state.typeIndentLevel,
				nextAction: state.nextAction
			};
		};

		this.equalStates = function equalStates(a, b) {
			return a.nextAction === b.nextAction &&
				a.typeIndentLevel === b.typeIndentLevel;
		};

	}).call(TestTaxonomy.prototype);

	Extensions.add('com.foldingtext.editor.taxonomies', {
		name: 'test',
		configurations: [
			{ mime: 'text/x-test', spec: 'test' }
		],
		taxonomy: function (spec) {
			return new TestTaxonomy(spec);
		}
	});

	exports.TestTaxonomy = TestTaxonomy;
});