define(function (require, exports, module) {
	'use strict';

	if (typeof Array.prototype.lastObject !== 'function') {
		Array.prototype.lastObject = function () {
			return this[this.length - 1];
		};
	}

	if (typeof Array.prototype.sortedIndex !== 'function') {
		Array.prototype.sortedIndex = function (value, sortfunction) {
			var low = 0, high = this.length;
			while (low < high) {
				/*jshint bitwise: false */
				var mid = (low + high) >> 1;
				/*jshint bitwise: true */
				var less = false;

				if (sortfunction !== undefined) {
					if (sortfunction(this[mid], value) < 0) {
						less = true;
					}
				} else {
					if (this[mid] < value) {
						less = true;
					}
				}

				if (less) {
					low = mid + 1;
				} else {
					high = mid;
				}
			}
			return low;
		};
	}


	if (typeof Array.prototype.shallowEquals !== 'function') {
		Array.prototype.shallowEquals = function (array) {
			if (this === array) { return true; }
			if (this.length !== array.length) { return false; }
			for (var i = 0; i < this.length; ++i) {
				if (this[i] !== array[i]) {
					return false;
				}
			}
			return true;
		};
	}

	if (typeof Array.prototype.sortedContains !== 'function') {
		Array.prototype.sortedContains = function (value, sortfunction) {
			return this[this.sortedIndex(value, sortfunction)] === value;
		};
	}

	if (typeof Array.prototype.sortedAdd !== 'function') {
		Array.prototype.sortedAdd = function (value, noDuplicates, sortfunction) {
			var index = this.sortedIndex(value, sortfunction);
			if (noDuplicates && this[index] === value) {
				return;
			}
			this.splice(index, 0, value);
		};
	}

	if (typeof Array.prototype.sortedRemove !== 'function') {
		Array.prototype.sortedRemove = function (value, sortfunction) {
			var index = this.sortedIndex(value, sortfunction);
			if (this[index] === value) {
				this.splice(index, 1);
			}
		};
	}

	if (typeof Array.prototype.sortedUniquify !== 'function') {
		Array.prototype.sortedUniquify = function (sortfunction) {
			var length = this.length,
				i = 0;
			while (i < length - 1) {
				var equal = false;
				if (sortfunction !== undefined) {
					equal = sortfunction(this[i], this[i + 1]) === 0;
				} else {
					if (this[i] === this[i + 1]) {
						equal = true;
					}
				}

				if (equal) {
					this.splice(i, 1);
					length--;
				} else {
					i++;
				}
			}
		};
	}
});