define(function (require, exports, module) {
	'use strict';

	function stopEvent(e) {
		if (e.preventDefault) {
			e.preventDefault();
		} else {
			e.returnValue = false;
		}

		if (e.stopPropagation) {
			e.stopPropagation();
		} else {
			e.cancelBubble = true;
		}
	}

	function addClass(element, classname) {
		if (element.classList) {
			element.classList.add(classname);
		}
		var cn = element.className;
		if (cn.indexOf(classname) !== -1) {
			return;
		}
		if (cn !== '') {
			classname = ' ' + classname;
		}
		element.className = cn + classname;
	}

	function removeClass(element, classname) {
		if (element.classList) {
			element.classList.remove(classname);
			return;
		}
		var cn = element.className;
		var rxp = new RegExp('\\s?\\b' + classname + '\\b', 'g');
		cn = cn.replace(rxp, '');
		element.className = cn;
	}

	function toggleClass(element, classname) {
		if (element.classList) {
			element.classList.toggle(classname);
		} else {
			var cn = element.className;
			var rxp = new RegExp('\\s?\\b' + classname + '\\b', 'g');
			if (rxp.test(cn)) {
				cn = cn.replace(rxp, '');
			} else {
				cn = [cn, classname].join(' ');
			}
			element.className = cn;
		}
	}

	function depthFirstNextNode(node, nodeType) {
		if (node.firstChild) { return node.firstChild; }
		if (node.nextSibling) { return node.nextSibling; }
		var p = node.parentElement;
		while (p) {
			var result = p.nextSibling;
			if (result) {
				if (nodeType === undefined || result.nodeType === nodeType) {
					return result;
				} else {
					return depthFirstNextNode(result, nodeType);
				}
			}
			p = p.parentElement;
		}
		return null;
	}

	function depthFirstPreviousNode(node, nodeType) {
		var prev = node.previousSibling,
			result;
		if (prev) {
			result = lastDescendantNodeOf(prev);
			if (nodeType === undefined || result.nodeType === nodeType) {
				return result;
			} else if (prev !== result) {
				return lastDescendantNodeOf(result);
			} else {
				return null;
			}
		} else {
			return node.parentElement;
		}
	}

	function lastDescendantNodeOf(node) {
		var last = node.lastChild;
		if (last) { return lastDescendantNodeOf(last); }
		return node;
	}

	return {
		stopEvent: stopEvent,
		addClass: addClass,
		removeClass: removeClass,
		toggleClass: toggleClass,
		depthFirstNextNode: depthFirstNextNode,
		depthFirstPreviousNode: depthFirstPreviousNode,
		lastDescendantNodeOf: lastDescendantNodeOf
	};
});