define(function (require, exports, module) {
	'use strict';

	if (typeof String.prototype.startsWith !== 'function') {
		String.prototype.startsWith = function (str) {
			return this.slice(0, str.length) === str;
		};
	}

	if (typeof String.prototype.endsWith !== 'function') {
		String.prototype.endsWith = function (str) {
			return this.slice(-str.length) === str;
		};
	}

	if (typeof String.prototype.replaceRange !== 'function') {
		String.prototype.replaceRange = function (start, end, substitute) {
			return this.substring(0, start) + substitute + this.substring(end);
		};
	}

	if (typeof String.prototype.trim !== 'function') {
		String.prototype.trim = function () {
			return this.replace(/(^\s*)|(\s*$)/gi, '');
		};
	}

	if (typeof String.prototype.ltrim !== 'function') {
		String.prototype.ltrim = function () {
			return this.replace(/^\s+/, '');
		};
	}

	if (typeof String.prototype.rtrim !== 'function') {
		String.prototype.rtrim = function () {
			return this.replace(/\s+$/, '');
		};
	}

	if (typeof String.prototype.hashCode !== 'function') {
		String.prototype.hashCode = function () {
			var hash = 0,
				length = this.length;
			
			if (length === 0) {
				return hash;
			}

			for (i = 0; i < length; i++) {
				char = this.charCodeAt(i);
				hash = ((hash << 5) - hash) + char;
				hash = hash & hash; // Convert to 32bit integer
			}

			return hash;		
		};
	}

	if (typeof String.prototype.escapeForRegex !== 'function') {
		// http://stackoverflow.com/questions/3446170/escape-string-for-use-in-javascript-regex
		String.prototype.escapeForRegex = function () {
			// Referring to the table here:
			// https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/regexp
			// these characters should be escaped
			// \ ^ $ * + ? . ( ) | { } [ ]
			// These characters only have special meaning inside of brackets
			// they do not need to be escaped, but they MAY be escaped
			// without any adverse effects (to the best of my knowledge and casual testing)
			// : ! , =
			// my test "~!@#$%^&*(){}[]`/=?+\|-_;:'\",<.>".match(/[\#]/g)

			var specials = [
					// order matters for these
					'-',
					'[',
					']',
					// order doesn't matter for any of these
					'/',
					'{',
					'}',
					'(',
					')',
					'*',
					'+',
					'?',
					'.',
					'\\',
					'^',
					'$',
					'|'
				],
				// I choose to escape every character with '\'
				// even though only some strictly require it when inside of []
				regex = new RegExp('[' + specials.join('\\') + ']', 'g');

			return this.replace(regex, '\\$&');
		};
	}
});