define(function (require) {
	'use strict';

	//	one
	//		two
	//			three
	//			four
	//		five
	//			six

	describe('Node', function () {
		var treeText = 'one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix',
			OutlineTaxonomy = require('ft/taxonomy/outlinetaxonomy').OutlineTaxonomy,
			root, one, two, three, four, five, six,
			Tree = require('ft/core/tree').Tree,
			tree;

		var NodeSet;

		beforeEach(function () {
			tree = new Tree(treeText, 'outline');
			tree.ensureClassified();
			root = tree.root;
			one = root.firstChild;
			two = one.nextLineNode();
			three = two.nextLineNode();
			four = three.nextLineNode();
			five = four.nextLineNode();
			six = five.nextLineNode();
		});

		afterEach(function () {
			tree.cleanupForCollection();
		});

		it('should set text', function () {
			one.setText('new text');
			expect(one.text()).toEqual('new text');
		});

		describe('Children', function () {

			it('should insert child in empty child list', function () {
				six.appendChild(tree.createNode('boo!'));
				expect(tree.text()).toEqual(treeText + '\n\t\t\tboo!');
			});

			it('should insert child before existing', function () {
				five.insertChildBefore(tree.createNode('boo!'), six);
				expect(tree.text()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tboo!\n\t\tsix');
			});

			it('should insert child after existing', function () {
				five.insertChildBefore(tree.createNode('boo!'), null);
				expect(tree.text()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix\n\t\tboo!');
			});

			it('should insert child before existing when existing is overindented', function () {
				six.setTypeIndentLevel(3);
				five.insertChildBefore(tree.createNode('boo!'), six);
				expect(tree.text()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\t\tboo!\n\t\t\tsix');
			});

			it('should insert child after existing when existing is overindented', function () {
				six.setTypeIndentLevel(3);
				five.insertChildBefore(tree.createNode('boo!'), null);
				expect(tree.text()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\t\tsix\n\t\t\tboo!');
			});

		});		
	});
});