define(function (require) {
	'use strict';

	describe('Taxonomy', function () {
		var Tree = require('ft/core/tree').Tree,
			taxonomy, node, tree;

		beforeEach(function () {
			tree = new Tree('Hello  world. Look at this.');
			node = tree.firstLineNode();
			taxonomy = tree.taxonomy;
		});

		afterEach(function () {
			tree.cleanupForCollection();
		});

		describe('Word Range', function () {
			it('should get word range from start of word', function () {
				expect(taxonomy.wordRange(node, 7)).toEqual({ location : 7, length : 5 });
			});

			it('should get word range from middle of word', function () {
				expect(taxonomy.wordRange(node, 9)).toEqual({ location : 7, length : 5 });
			});

			it('should get word range from end of word', function () {
				expect(taxonomy.wordRange(node, 12)).toEqual({ location : 7, length : 5 });
			});

			it('should get word range from middle of multiple whitespaces', function () {
				expect(taxonomy.wordRange(node, 6)).toEqual({ location : 5, length : 2 });
			});

			it('should get word range from non assci whitespace separated words', function () {
				node.setLine('규격 착오');
				expect(taxonomy.wordRange(node, 0)).toEqual({ location : 0, length : 2 });
			});
		});

		describe('Sentence Range', function () {
		});

		describe('Modification', function () {

			//
			// The default taxonomy allows you to modify tree, but doesn't encode any of your changes into
			// the text content of the tree since the default taxonomy specifies no syntax.
			//

			it('should set type', function () {
				node.setType('test');
				expect(node.type()).toEqual('test');
				expect(node.line()).toEqual('Hello  world. Look at this.');
			});

			it('should set type indent level', function () {
				node.setTypeIndentLevel(20);
				expect(node.typeIndentLevel()).toEqual(20);
				expect(node.line()).toEqual('Hello  world. Look at this.');
			});

			it('should set mode', function () {
				node.setMode('boo!');
				expect(node.mode()).toEqual('boo!');
				expect(node.line()).toEqual('Hello  world. Look at this.');
			});

			it('should add tag', function () {
				node.addTag('tag', 'value');
				expect(node.tags()).toEqual({ tag : 'value' });
				expect(node.line()).toEqual('Hello  world. Look at this.');
			});

			it('should add child', function () {
				var child = tree.createNode('new child');
				node.appendChild(child);
				expect(node.firstChild).toEqual(child);
				expect(child.parent).toEqual(node);
				expect(child.typeIndentLevel()).toEqual(1);
				expect(node.line()).toEqual('Hello  world. Look at this.');
				expect(child.line()).toEqual('new child');
			});

			it('should remove child', function () {
				var child = tree.createNode('new child');
				node.appendChild(child);
				node.removeChild(child);
				expect(node.firstChild).toEqual(null);
				expect(child.parent).toEqual(null);
				expect(child.typeIndentLevel()).toEqual(1);
				expect(node.line()).toEqual('Hello  world. Look at this.');
				expect(child.line()).toEqual('new child');
			});
		});
	});
});