define(function (require) {
	'use strict';

	describe('Editor', function () {
		var Editor = require('ft/editor/editor').Editor,
			editor,
			tree,
			cm;

		beforeEach(function () {
			editor = new Editor('');
			tree = editor.tree();
			cm = editor.cm();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		it('should created with default value', function () {
			expect(editor.textContent()).toEqual('');
		});

		it('should set and get text content', function () {
			editor.setTextContent('Hello world!');
			expect(editor.textContent()).toEqual('Hello world!');
		});

		describe('Sync', function () {
			it('should update CM when modifying Tree', function () {
				tree.setText('one\ntwo\n\tthree');
				expect(cm.getValue()).toEqual('one\ntwo\n\tthree');

				tree.removeNode(tree.root.firstChild.nextSibling);
				expect(cm.getValue()).toEqual('one\n\tthree');

				tree.root.firstChild.replaceLineInRange('zz', 1, 1);
				expect(cm.getValue()).toEqual('ozze\n\tthree');

				tree.insertNodeBefore(tree.createNode('new1'), tree.root.firstChild);
				expect(cm.getValue()).toEqual('new1\nozze\n\tthree');

				tree.insertNodeBefore(tree.createNode('new2'), tree.root.firstChild.nextSibling);
				expect(cm.getValue()).toEqual('new1\nnew2\nozze\n\tthree');
			});

			it('should update Tree when modifying CM', function () {
				cm.setValue('one\ntwo\n\tthree');
				expect(tree.text()).toEqual('one\ntwo\n\tthree');
				cm.replaceRange('', {line: 1, ch: 0}, {line: 2, ch: 0});
				expect(tree.text()).toEqual('one\n\tthree');
			});

			it('should update Tree when undoing CM', function () {
				cm.setValue('one\ntwo\n\tthree');
				expect(tree.text()).toEqual('one\ntwo\n\tthree');
				cm.undo();
				expect(tree.text()).toEqual('');
			});

		});

		describe('Extensions', function () {
			it('should allow options', function () {
			});

			it('should allow init hooks', function () {
			});

			it('should allow defined extensions', function () {
			});
		});

		describe('Keymaps', function () {
		});
	});
});