define(function (require) {
	'use strict';

	describe('Current', function () {
		require('ft/extensions/current');

		var	Extensions = require('ft/core/extensions').Extensions,
			Editor = require('ft/editor/editor').Editor,
			editor,
			currentWord,
			updateCurrentWord = function (editor, range) {
				currentWord = range ? range.textInRange() : null;
			};

		beforeEach(function () {
			jasmine.addMatchers(require('tests/custommatchers'));
			Extensions.add('com.foldingtext.editor.currentWordDidChange', updateCurrentWord);
			editor = new Editor('');
		});

		afterEach(function () {
			Extensions.remove('com.foldingtext.editor.currentWordDidChange', function (each) { return each === updateCurrentWord; });
			editor.removeAndCleanupForCollection();
			currentWord = null;
		});

		it('should mark current after init', function () {
			editor.removeAndCleanupForCollection();
			editor = new Editor('one two\ntwo three\nthree four');
			expect(editor._currentWordMark.find()).toBeJsonEqual({ from : { line : 0, ch : 0 }, to : { line : 0, ch : 3 } });
			expect(editor._currentSentenceMark.find()).toBeJsonEqual({ from : { line : 0, ch : 0 }, to : { line : 0, ch : 7 } });
			expect(editor.currentWordRange().textInRange()).toEqual('one');
			expect(currentWord).toEqual('one');
		});

		it('should mark current', function () {
			editor.setTextContent('one two\ntwo three\nthree four');
			expect(editor._currentWordMark.find()).toBeJsonEqual({ from : { line : 0, ch : 0 }, to : { line : 0, ch : 3 } });
			expect(editor._currentSentenceMark.find()).toBeJsonEqual({ from : { line : 0, ch : 0 }, to : { line : 0, ch : 7 } });
			expect(editor.currentWordRange().textInRange()).toEqual('one');
			expect(currentWord).toEqual('one');
		});

		it('should upate current', function () {
			editor.setTextContent('one two\ntwo three\nthree four');
			editor.setSelectedLineCh({line: 1, ch: 0});
			expect(editor._currentWordMark.find()).toBeJsonEqual({ from : { line : 1, ch : 0 }, to : { line : 1, ch : 3 } });
			expect(editor._currentSentenceMark.find()).toBeJsonEqual({ from : { line : 1, ch : 0 }, to : { line : 1, ch : 9 } });
			expect(currentWord).toEqual('two');
		});

		it('should upate current if insert space in front', function () {
			editor.setTextContent('one two\ntwo three\nthree four');
			editor.setSelectedLineCh({line: 1, ch: 0});
			currentWord = null;
			editor.replaceSelection(' ', 'end', '+input');
			expect(currentWord).toEqual('two');
		});
	});
});