define(function (require) {
	'use strict';

/*
	one
		two
			three
			four
		five
			six
*/

	describe('Focus', function () {

		require('ft/extensions/focus');
		require('ft/extensions/folding');
		require('ft/extensions/foldingnodes');
		require('ft/taxonomy/outlinetaxonomy');

		var treeText = 'one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix',
			Editor = require('ft/editor/editor').Editor,
			one, two, three, four, five, six,
			Tree = require('ft/core/tree').Tree,
			editor, tree;

		beforeEach(function () {
			editor = new Editor(treeText, 'outline');
			tree = editor.tree();
			one = tree.root.firstChild;
			two = one.nextLineNode();
			three = two.nextLineNode();
			four = three.nextLineNode();
			five = four.nextLineNode();
			six = five.nextLineNode();
			tree.ensureClassified();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		it('should focus focusRange', function () {
			editor.setSelectedLineCh({line: 1, ch: 0});
			editor.focusRange(tree.createRangeFromLocation(1, 3));
			expect(editor.folds().length).toEqual(2);
		});

		it('should focus node', function () {
			editor.focusNode(two);
			expect(editor.folds().length).toEqual(2);
			editor.focusNode(five);
			expect(editor.folds().length).toEqual(1);
			editor.focusNode(one);
			expect(editor.folds().length).toEqual(0);
		});

		it('should maintain folds in focus region while editing text', function () {
			editor.setSelectedLineCh({line: 4, ch: 5});
			editor.performCommand('focusIn');
			expect(editor.folds().length).toEqual(1);
			editor.performCommand('smartInsertNewline');
			expect(editor.textContent()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\t\n\t\tsix');
			expect(editor.folds().length).toEqual(1);
		});
	});
});