define(function (require) {
	'use strict';

/*
	one
		two
			three
			four
		five
			six
*/

	describe('Indent', function () {

		require('ft/taxonomy/outlinetaxonomy');

		var Folding = require('ft/extensions/folding'),
			Indent = require('ft/extensions/indent'),
			treeText = 'one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix',
			Editor = require('ft/editor/editor').Editor,
			one, two, three, four, five, six,
			Tree = require('ft/core/tree').Tree,
			editor, tree;

		beforeEach(function () {
			editor = new Editor(treeText, 'outline', {indentWithTabs: true, indentUnit: 4});
			tree = editor.tree();
			one = tree.root.firstChild;
			two = one.nextLineNode();
			three = two.nextLineNode();
			four = three.nextLineNode();
			five = four.nextLineNode();
			six = five.nextLineNode();
			tree.ensureClassified();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		it('should indent more', function () {
			editor.setSelectedLineCh({line: 0, ch: 0});
			editor.performCommand('moveParagraphsRight');
			expect(editor.textContent()).toEqual('\tone\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix');
		});

		it('should indent less', function () {
			editor.setSelectedLineCh({line: 2, ch: 0});
			editor.performCommand('moveParagraphsLeft');
			expect(editor.textContent()).toEqual('one\n\ttwo\n\tthree\n\t\tfour\n\tfive\n\t\tsix');
		});
	});
});