/*global XMLSerializer:true */
define(function (require) {
	'use strict';

	describe('OPML Export', function () {
		require('ft/taxonomy/outlinetaxonomy');

		var treeText = 'one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix',
			Editor = require('ft/editor/editor').Editor,
			one, two, three, four, five, six,
			Tree = require('ft/core/tree').Tree,
			OPMLTransform = require('ft/extensions/opml').OPMLTransform,
			editor, tree, opmlTransformer;

		beforeEach(function () {
			editor = new Editor(treeText, 'outline');
			tree = editor.tree();
			opmlTransformer = new OPMLTransform(tree);
			one = tree.root.firstChild;
			two = one.nextLineNode();
			three = two.nextLineNode();
			four = three.nextLineNode();
			five = four.nextLineNode();
			six = five.nextLineNode();
			tree.ensureClassified();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		it('should create a fully formed OPML Document from a single node', function () {
			var opml = opmlTransformer.toOPMLObject(four);

			var result = [
				'<opml version="2.0">',
				'<head/>',
				'<body>',
				'<outline text="four" type="item" level="2"/>',
				'</body>',
				'</opml>'
			].join('');

			var opmlString = (new XMLSerializer()).serializeToString(opml);
			expect(opmlString).toEqual(result);
		});

		it('should work fine with sensitive text stuff', function () {
			tree.setText('"people" <dont />\n\tjibberish #@&$%^&');
			var opml = opmlTransformer.toOPMLObject();
			var result = [
				'<opml version="2.0">',
				'<head/>',
				'<body>',
				'<outline text="&quot;people&quot; &lt;dont /&gt;" type="item" level="0">',
				'<outline text="jibberish #@&amp;$%^&amp;" type="item" level="1"/>',
				'</outline>',
				'</body>',
				'</opml>'
			].join('');

			var opmlString = (new XMLSerializer()).serializeToString(opml);
			expect(opmlString).toEqual(result);
		});

		/*it('should be able to parse an opml document passed as a string', function () {
			var docAsOPML = (new XMLSerializer()).serializeToString(opmlTransformer.toOPMLObject());
			tree.setText('');

			expect(tree.text()).toEqual('');

			opmlTransformer.loadDocumentFromOPMLString(docAsOPML, null, true);

			expect(tree.text()).toEqual([
				'one',
				'\ttwo',
				'\t\tthree',
				'\t\tfour',
				'\tfive',
				'\t\tsix'
			].join('\n'));
		});

		it('should be able to merge an opml string within an existing document', function () {
			var docAsOPML = (new XMLSerializer()).serializeToString(opmlTransformer.toOPMLObject(two));
			var oldText = tree.text();

			tree.replaceTextInRange(tree.createRangeFromNodes(two, 0, five, 0), '');

			expect(tree.text()).toEqual([
				'one',
				'\tfive',
				'\t\tsix'
			].join('\n'));

			opmlTransformer.loadDocumentFromOPMLString(docAsOPML, tree.lineNumberToNode(1), false);

			expect(tree.text()).toEqual(oldText);
		});*/
	});
});