define(function (require) {
    'use strict';

    /*xdescribe('Panel', function () {
        require('ft/extensions/panel');

        var Editor = require('ft/editor/editor').Editor,
            editor,
            panel;

        beforeEach(function () {
            editor = new Editor('');
            panel = editor.panel();
            panel.show('bottom');
        });

        afterEach(function () {
            editor.removeAndCleanupForCollection();
        });

        it('should have a panel present', function () {
            expect(panel._panelElement).toBeTruthy();
            expect(panel._panelElement.parentElement).toBeTruthy();
            expect(panel._panelEditor.getValue()).toEqual('');
        });

        it('should be able to add a field', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'This is simple: '
            }]);

            expect(panel.fieldEditor().getValue()).toEqual('This is simple: ');

            expect(panel._fields.length).toEqual(1);
            expect(panel._fields[0].name).toEqual('simple');
            expect(panel._fields[0].marker.find().from.ch).toEqual(0);
            expect(panel._fields[0].marker.find().to.ch).toEqual(15);
        });

        it('should be able to add multiple fields', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'This is simple: \n',
                markerLength: 15
            }, {
                name: 'simpler',
                value: 'This is even simpler: ',
                index: 1
            }]);

            expect(panel.fieldEditor().getValue()).toEqual('This is simple: \nThis is even simpler: ');
            expect(panel._fields.length).toEqual(2);

            expect(panel._fields[0].name).toEqual('simple');
            expect(panel._fields[0].marker.find().from.line).toEqual(0);
            expect(panel._fields[0].marker.find().from.ch).toEqual(0);
            expect(panel._fields[0].marker.find().to.ch).toEqual(15);

            expect(panel._fields[1].name).toEqual('simpler');
            expect(panel._fields[1].marker.find().from.line).toEqual(1);
            expect(panel._fields[1].marker.find().from.ch).toEqual(0);
            expect(panel._fields[1].marker.find().to.ch).toEqual(21);
        });

        it('should query the value of a field', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'SIM: \n',
                markerLength: 4
            }, {
                name: 'simpler',
                value: 'SIM2: ',
                index: 1
            }]);

            panel.fieldEditor().replaceRange('value', {line: 0, ch: 5});

            expect(panel.fieldValue('simple')).toEqual('value');

            panel.fieldEditor().replaceRange('value2', {line: 1, ch: 6});

            expect(panel.fieldValue('simple')).toEqual('value');
            expect(panel.fieldValue('simpler')).toEqual('value2');
        });

        it('should callback field changes', function () {
            var owner = {};
            owner.fieldValueChanged = jasmine.createSpy('callback');
            CodeMirror.on(panel, 'fieldvaluechanged', owner.fieldValueChanged);

            panel.addFields(owner, [{
                name: 'simple',
                value: 'SIM: \n',
                markerLength: 4
            }, {
                name: 'simpler',
                value: 'SIM2: ',
                index: 1
            }]);

            panel.fieldEditor().replaceRange('value', {line: 0, ch: 5});

            expect(panel.fieldValue('simple')).toEqual('value');
            expect(owner.fieldValueChanged).toHaveBeenCalledWith({
                from: '',
                to: 'value',
                name: 'simple'
            });

            panel.fieldEditor().replaceRange('value2', {line: 1, ch: 6});
            expect(owner.fieldValueChanged).toHaveBeenCalledWith({
                from: '',
                to: 'value2',
                name: 'simpler'
            });
        });

        it('should be able to remove fields', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'This is simple: '
            }]);

            expect(panel.fieldEditor().getValue()).toEqual('This is simple: ');

            expect(panel._fields.length).toEqual(1);
            expect(panel._fields[0].name).toEqual('simple');
            expect(panel._fields[0].marker.find().from.ch).toEqual(0);
            expect(panel._fields[0].marker.find().to.ch).toEqual(15);

            panel.removeField('simple');

            expect(panel.hasField('simple')).toBeFalsy();
            expect(panel._fields.length).toEqual(0);
            expect(panel.fieldEditor().getValue()).toEqual('');
        });

        it('should be able to remove a field among others', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'SIM: \n',
                markerLength: 4
            }, {
                name: 'simpler',
                value: 'SIM2: ',
                index: 1
            }]);

            panel.fieldEditor().replaceRange('value', {line: 0, ch: 5});

            expect(panel.fieldValue('simple')).toEqual('value');

            panel.fieldEditor().replaceRange('value2', {line: 1, ch: 6});

            expect(panel.fieldValue('simple')).toEqual('value');
            expect(panel.fieldValue('simpler')).toEqual('value2');

            panel.removeField('simple');

            expect(panel.hasField('simple')).toBeFalsy();
            expect(panel._fields.length).toEqual(1);
            expect(panel.fieldValue('simpler')).toEqual('value2');
            expect(panel.fieldValue('simple')).toBeNull();
            expect(panel.fieldEditor().getValue()).toEqual('SIM2: value2');
        });

        it('should remove and re-add fields without any extra new lines', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'SIM: ',
                markerLength: 4
            }, {
                name: 'simpler',
                value: '\nSIM2: ',
                index: 1,
                markerStart: 1,
                markerLength: 5
            }]);

            panel.fieldEditor().replaceRange('value', {line: 0, ch: 5});

            expect(panel.fieldValue('simple')).toEqual('value');

            panel.fieldEditor().replaceRange('value2', {line: 1, ch: 6});

            expect(panel.fieldValue('simple')).toEqual('value');
            expect(panel.fieldValue('simpler')).toEqual('value2');

            panel.removeField('simpler');

            expect(panel.hasField('simpler')).toBeFalsy();
            expect(panel.fieldEditor().getValue()).toEqual('SIM: value');

            panel.addFields(owner, [{
                name: 'simpler',
                value: '\nSIM2: ',
                index: 1,
                markerStart: 1,
                markerLength: 5
            }]);

            expect(panel.fieldEditor().getValue()).toEqual('SIM: value\nSIM2: ');

            panel.removeField('simple');

            expect(panel.fieldEditor().getValue()).toEqual('SIM2: ');
        });

        it('should update field value depending on changes', function () {
            var owner = {};
            panel.addFields(owner, [{
                name: 'simple',
                value: 'SIM: ',
                markerLength: 4
            }, {
                name: 'simpler',
                value: '\nSIM2: ',
                index: 1,
                markerStart: 1,
                markerLength: 5
            }]);

            expect(panel.fieldValue('simple')).toEqual('');
            expect(panel.fieldValue('simpler')).toEqual('');

            panel.fieldEditor().replaceRange('value', {line: 0, ch: 5});

            expect(panel.fieldValue('simple')).toEqual('value');
        });
    });*/
});