define(function (require) {
	'use strict';

/*
	one
		two
			three
			four
		five
			six
*/

	describe('Smart Newline', function () {

		require('ft/taxonomy/outlinetaxonomy');

		var Extensions = require('ft/core/extensions').Extensions,
			Folding = require('ft/extensions/folding'),
			SmartNewline = require('ft/extensions/smartnewline'),
			treeText = 'one\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix',
			Editor = require('ft/editor/editor').Editor,
			one, two, three, four, five, six,
			Tree = require('ft/core/tree').Tree,
			editor, tree;

		beforeEach(function () {
			editor = new Editor(treeText, 'outline');
			tree = editor.tree();
			one = tree.root.firstChild;
			two = one.nextLineNode();
			three = two.nextLineNode();
			four = three.nextLineNode();
			five = four.nextLineNode();
			six = five.nextLineNode();
			tree.ensureClassified();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		it('should match child level of first child if it exists', function () {
			editor.setSelectedLineCh({line: 0, ch: 3});
			editor.performCommand('smartInsertNewline');
			expect(editor.textContent()).toEqual('one\n\t\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix');
		});

		it('should match level of current if no child exists', function () {
			editor.setSelectedLineCh({line: 2, ch: 7});
			editor.performCommand('smartInsertNewline');
			expect(editor.textContent()).toEqual('one\n\ttwo\n\t\tthree\n\t\t\n\t\tfour\n\tfive\n\t\tsix');
		});

		it('should match level of first previous visible, if current isn\'t visible', function () {
			editor.setSelectedLineCh({line: 1, ch: 0});
			editor.performCommand('toggleNodeFold');
			editor.setSelectedLineCh({line: 3, ch: 6});
			editor.performCommand('smartInsertNewline');
			expect(editor.textContent()).toEqual('one\n\ttwo\n\t\tthree\n\t\tfour\n\t\n\tfive\n\t\tsix');
		});

		it('should insert newline at start (and not expand) if newline inserted in indented region', function () {
			editor.setSelectedLineCh({line: 1, ch: 1});
			editor.performCommand('toggleNodeFold');
			editor.performCommand('smartInsertNewline');
			expect(editor.isCollapsed(two.nextLineNode())).toBeTruthy();
			expect(editor.textContent()).toEqual('one\n\n\ttwo\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix');
		});

		it('should expand fold when inserting newline before folded region', function () {
			editor.setSelectedLineCh({line: 1, ch: 4});
			editor.performCommand('toggleNodeFold');
			expect(editor.isCollapsed(two)).toBeTruthy();
			editor.performCommand('smartInsertNewline');
			expect(editor.isCollapsed(two)).toBeFalsy();
			expect(editor.textContent()).toEqual('one\n\ttwo\n\t\t\n\t\tthree\n\t\tfour\n\tfive\n\t\tsix');
		});
	});
});