define(function (require) {
	'use strict';

	describe('Snippets', function () {
		require('ft/extensions/snippets');

		var Extensions = require('ft/core/extensions').Extensions,
			Editor = require('ft/editor/editor').Editor,
			editor,
			snippets = {
				'abc' : 'aabbcc',
				'...' : '…'
			};

		beforeEach(function () {
			Extensions.add('com.foldingtext.editor.snippets', snippets);
			editor = new Editor('');
		});

		afterEach(function () {
			Extensions.remove('com.foldingtext.editor.snippets', function (each) { return each === snippets; });
			editor.removeAndCleanupForCollection();
		});

		it('should expand snippet', function () {
			editor.setTextContent('abc');
			editor.setSelectedLineCh({line: 0, ch: 3});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual('aabbcc');
		});

		it('should always expand snippet starting with non word char', function () {
			editor.setTextContent(' ...');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual(' …');

			editor.setTextContent('....');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual('.…');

			editor.setTextContent('a...');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual('a…');
		});

		it('should only expand snippet starting with word char if previous char is not word', function () {
			editor.setTextContent(' abc');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual(' aabbcc');

			editor.setTextContent('.abc');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual('.aabbcc');

			editor.setTextContent('mabc');
			editor.setSelectedLineCh({line: 0, ch: 4});
			editor.performCommand('replaceTextSnippet');
			expect(editor.textContent()).toEqual('mabc');
		});
	});
});