define(function (require) {
	'use strict';

	describe('Text Checking', function () {
		require('ft/extensions/textchecking');

		var Editor = require('ft/editor/editor').Editor,
			editor,
			tree,
			node;

		beforeEach(function () {
			editor = new Editor('teh');
			tree = editor.tree();
			node = tree.firstLineNode();
		});

		afterEach(function () {
			editor.removeAndCleanupForCollection();
		});

		function waitForTree(test, then) {
			if (test()) {
				then();
			} else {
				var listener = function (e) {
					if (test()) {
						tree.removeEventListener('treeChanged', listener);
						then();
					}
				};
				tree.addEventListener('treeChanged', listener);
			}
		}

		it('should check "teh" as spelling error', function (done) {
			waitForTree(
				function test() {
					return node.lineAttributedString().toString(true) === '(teh/spelling="teh")';
				},
				function then() {
					done();
				}
			);
		});

		it('should immediatly remove check "teh" as spelling error when word is edited', function (done) {
			waitForTree(
				function test() {
					return node.lineAttributedString().toString(true) === '(teh/spelling="teh")';
				},
				function then() {
					window.setTimeout(function () {
						editor.setSelectedRange(tree.createRangeFromLocation(1, 0));
						editor.replaceSelection('b', 'end', '+input');
						expect(node.lineAttributedString().toString(true)).toEqual('(tbeh/)');
						done();
					});
				}
			);
		});

		it('should immeditaly not extend check "teh" as spelling error when space is added after word', function (done) {
			waitForTree(
				function test() {
					return node.lineAttributedString().toString(true) === '(teh/spelling="teh")';
				},
				function then() {
					window.setTimeout(function () {
						editor.setSelectedRange(tree.createRangeFromLocation(3, 0));
						editor.replaceSelection(' ', 'end', '+input');
						expect(node.lineAttributedString().toString(true)).toEqual('(teh/spelling="teh")( /)');
						done();
					});
				}
			);
		});

		it('should immeditaly not extend check "Затеивать" as spelling error when space is added after word', function (done) {
			waitForTree(
				function test() {
					return node.lineAttributedString().toString(true) === '(Затеивать/spelling="Затеивать")';
				},
				function then() {
					window.setTimeout(function () {
						editor.setSelectedRange(tree.createRangeFromLocation(9, 0));
						editor.replaceSelection(' ', 'end', '+input');
						expect(node.lineAttributedString().toString(true)).toEqual('(Затеивать/spelling="Затеивать")( /)');
						done();
					});
				}
			);
			node.setLine('Затеивать');
		});

		it('should immeditaly not extend check "teh" as spelling error when space is added before word', function (done) {
			waitForTree(
				function test() {
					return node.lineAttributedString().toString(true) === '(teh/spelling="teh")';
				},
				function then() {
					window.setTimeout(function () {
						editor.replaceSelection(' ', 'end', '+input');
						expect(node.lineAttributedString().toString(true)).toEqual('( /)(teh/spelling="teh")');
						done();
					});
				}
			);
		});
	});
});