define(function (require) {
	'use strict';

	describe('MultiMarkdown Taxonomy', function () {
		var MarkdownTaxonomy = require('ft/taxonomy/markdowntaxonomy').MarkdownTaxonomy,
			Taxonomies = require('ft/core/taxonomies'),
			Tree = require('ft/core/tree').Tree,
			tree;

		beforeEach(function () {
			tree = new Tree('', Taxonomies.taxonomy({
				multimarkdown: true
			}, 'markdown'));
		});

		afterEach(function () {
			tree.cleanupForCollection();
		});

	/*	describe('Metadata Type', function () {
			it('should allow run of metadata at start of document until empty line', function () {
				tree.setText('my: metadata');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('metadata');
			});
		});
*/
		describe('Link Definition Type', function () {
			it('should allow trailing attributes after title', function () {
				tree.setText('[one]: two "title" class=mouse');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('linkdefinition');
				expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('([/keyword, text)(one/linkid, text)(]:/keyword, text)( /text)(two/linktarget, text)( /text)("/keyword, text)(title/linktitle, text)("/keyword, text)( class=mouse/text)');
			});

			it('should allow footnote style link definitions', function () {
				tree.setText('[^one]: two');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('footnotelinkdefinition');
				expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('([^/keyword, text)(one/linkid, text)(]:/keyword, text)( two/text)');
			});
		});

		it('should detect label after heading text', function () {
			tree.setText('# one [two]');
			tree.classifier.ensureClassified();
			expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('(#/keyword, lead)( /lead)(one/text)( /)([/keyword, label)(two/label)(]/keyword, label)');

			tree.setText('# one [two] #');
			tree.classifier.ensureClassified();
			expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('(#/keyword, lead)( /lead)(one/text)( /)([/keyword, label)(two/label)(]/keyword, label)( /)(#/keyword)');

			tree.setText('# one    [two]    #    ');
			tree.classifier.ensureClassified();
			expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('(#/keyword, lead)( /lead)(one/text)(    /)([/keyword, label)(two/label)(]/keyword, label)(    /)(#/keyword)(    /)');
		});

		describe('Definition List Type', function () {
			it('should detect definition lists', function () {
				tree.setText('one\n: two');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('term');
				expect(tree.lastLineNode().type()).toEqual('definition');
				expect(tree.lastLineNode().text()).toEqual('two');
			});

			it('should not detect definition with leading spaces', function () {
				tree.setText('one\n : two');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).not.toEqual('term');
				expect(tree.lastLineNode().type()).not.toEqual('definition');
				expect(tree.lastLineNode().text()).not.toEqual('two');
			});

			it('should detect definition lists with multiple terms', function () {
				tree.setText('one\ntwo\n: three');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('term');
				expect(tree.firstLineNode().nextLineNode().type()).toEqual('term');
				expect(tree.lastLineNode().type()).toEqual('definition');
				expect(tree.lastLineNode().text()).toEqual('three');
			});

			it('should detect definition lists with multiple definitions', function () {
				tree.setText('one\n: two\n: three');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().type()).toEqual('term');
				expect(tree.firstLineNode().nextLineNode().type()).toEqual('definition');
				expect(tree.lastLineNode().type()).toEqual('definition');
				expect(tree.lastLineNode().text()).toEqual('three');
			});
		});

		describe('Inline', function () {
			it('should detect inline footnote links', function () {
				tree.setText('I have a [^footnote].');
				tree.classifier.ensureClassified();
				expect(tree.firstLineNode().lineAttributedString().toString()).toEqual('(I have a /text)([^/footnotelink, keyword, text)(footnote/footnotelink, linklabel, text)(]/footnotelink, keyword, text)(./text)');
			});
		});
	});
});