define(function (require) {
	'use strict';

	describe('TaskPaper Taxonomy', function () {
		var TaskPaperTaxonomy = require('ft/taxonomy/taskpapertaxonomy').TaskPaperTaxonomy,
			Extensions = require('ft/core/extensions').Extensions,
			Taxonomies = require('ft/core/taxonomies'),
			Tree = require('ft/core/tree').Tree,
			one, two, three, four, five, six,
			tree;

		beforeEach(function () {
			tree = new Tree('', Taxonomies.taxonomy({}, 'taskpaper'));
		});

		afterEach(function () {
			tree.cleanupForCollection();
		});

		it('should recognize projects', function () {
			tree.setText('my project: @important');
			tree.classifier.ensureClassified();
			var line = tree.firstLineNode();
			expect(line.type()).toEqual('project');
			expect(line.lineAttributedString().toString()).toEqual('(my project/text)(:/keyword)( /)(@/keyword, tag, trailingtags)(important/tag, tagname, trailingtags)');
		});

		it('should recognize tasks', function () {
			tree.setText('- my task @important');
			tree.classifier.ensureClassified();
			var line = tree.firstLineNode();
			expect(line.type()).toEqual('task');
			expect(line.lineAttributedString().toString()).toEqual('(-/keyword, lead)( /lead)(my task/text)( /)(@/keyword, tag, trailingtags)(important/tag, tagname, trailingtags)');
		});

		it('should recognize notes', function () {
			tree.setText('my note @important');
			tree.classifier.ensureClassified();
			var line = tree.firstLineNode();
			expect(line.type()).toEqual('note');
			expect(line.lineAttributedString().toString()).toEqual('(my note/text)( /)(@/keyword, tag, trailingtags)(important/tag, tagname, trailingtags)');
		});

		it('should calculate proper trailing tags range', function () {
			tree.setText('my note   @important ');
			tree.classifier.ensureClassified();
			var line = tree.firstLineNode();
			expect(line.type()).toEqual('note');
			expect(line.lineAttributedString().toString()).toEqual('(my note/text)(   /)(@/keyword, tag, trailingtags)(important/tag, tagname, trailingtags)( /trailingtags)');
		});

		it('should use tabs for structure', function () {
			tree.setText('project:\n\t- task 1\n\tnote 1\n\t- task 2');
			tree.classifier.ensureClassified();
			var project = tree.firstLineNode(),
				task1 = project.nextLineNode(),
				note1 = task1.nextLineNode(),
				task2 = note1.nextLineNode();

			expect(project.firstChild).toEqual(task1);
			expect(project.lastChild).toEqual(task2);
			expect(task1.nextSibling).toEqual(note1);
			expect(note1.nextSibling).toEqual(task2);
		});
	});
});